class ApplicationController < RxModule::ApplicationController
  include BadDataLogger

  skip_before_action :verify_authenticity_token
  before_action :set_default_response_format
  after_action :set_rate_limit_headers
  rescue_from ActionDispatch::Http::Parameters::ParseError, with: :unprocessable_entity

  def time
    @time = {
      timestamp: RxModule::Time.now,
      locale: I18n.locale
    }
    render json: @time
  end

  protected

  def write_error(e, request)
    request.body.rewind
    write([e.message, e.backtrace, "URL: #{request.fullpath}", "BODY: #{request.body.read}"])
  end

  def set_rate_limit_headers
    add_rate_limit_headers("req/ip")
  end

  def unprocessable_entity(exception)
    Rails.logger.warn "Failed to parse request #{request.fullpath}. Request body: #{request.body.read}"
    write_error(exception, request)
    head :unprocessable_entity
  end

  def set_default_response_format
    request.format = "json"
  end

  def enum_value(input)
    int = Integer(input, exception: false)
    int || input.downcase
  end
end
