class BatchesController < ApplicationController
  before_action :block_index_all, only: [:index]

  model_data_store(Batch,
                   {create: true, read: true, update: false, delete: false, index: true},
                   {create: true, read: true, update: false, delete: false, index: true})

  # Override ApplicationController set_target
  def set_target
    model = Batch.find_by_id(params[:id])
    if model.nil?
      ActiveRecord::Base.connected_to(shard: :archive, role: :reading) do
        model = Batch.find_by_id(params[:id])
      end
    end
    @model = model
  end

  def create
    params = json_to_batch_params
    AddBatch.perform_later(params[:workstationId], params[:batch])
    render json: {status: 'success'}, status: :ok
  end

  def batch_data_as_json
    batch_id = params[:batch_id]
    batch_to_json(batch_id)
  end

  # Called using a job number, job or batch could be in primary or archive shard
  def search_by_job_number
    process_code = params[:process_code]
    job_number = params[:job_number]
    batch = ActiveRecord::Base.connected_to(shard: :primary, role: :reading) do
      batch_for_process_code_and_job_number(process_code, job_number)
    end
    return render json: { batch_id: batch.id }, status: :ok unless batch.nil?
    batch = ActiveRecord::Base.connected_to(shard: :archive, role: :reading) do
      batch_for_process_code_and_job_number(process_code, job_number)
    end
    return render json: { batch_id: batch.id }, status: :ok unless batch.nil?
    render json: { error: "Batch not found" }, status: :unprocessable_entity
  end

  private

  def batch_for_process_code_and_job_number(process_code, job_number)
    Job.joins(batch_job_data: { batch_container: :batch })
       .select("batch.*")
       .where(number: job_number)
       .where(batch: { :process_code => process_code })
       .order("batch.created_at DESC")
       .first
  end

  def batch_to_json(batch_id)
    batch = Batch.find_by_batch_id(batch_id)
    return render json: batch.get_json, status: :ok unless batch.nil?
    ActiveRecord::Base.connected_to(shard: :archive, role: :reading) do
      batch = Batch.find_by_batch_id(batch_id)
      return render json: batch.get_json, status: :ok unless batch.nil?
    end
    render json: { error: 'Batch does not exist' }, status: :unprocessable_entity
  end

  def json_to_batch_params
    params.permit(:workstationId, :batch => {})
  end

  # Only allow a list of trusted parameters through.
  def model_params
    params.require(:batch).permit(:batch_id, :process_code, :created_at, :updated_at)
  end
end
