class ErrorsController < ApplicationController
  include MachineResponse
  include MachineDataViewFilter
  include RxModule::Tableable

  wrap_parameters :problem, include: [:piececounter, :errorno, :error, :errortype, :status, :job, :process, :lensside, :date, :time, :sn]
  model_data_store(Error)

  # get /error/1.0
  # params:
  #   startTime
  #   endTime
  #   offset
  #   limit
  #   sn
  #   errorType
  #   status
  def errors
    ActiveRecord::Base.connected_to(shard: :archive, role: :reading) do
      start_time = Time.zone.parse(params[:startTime] || "") || Time.at(0)
      end_time = Time.zone.parse(params[:endTime] || "")  || Time.current
      @offset = params[:offset] || 0
      @limit = params[:limit] || 200

      @machines = Machine.all
      @machines = @machines.where(sn: params[:sn]) if params[:sn]

      @errors = Error.where(time: start_time..end_time)
      @errors = @errors.where(machine_sn: @machines.pluck(:sn))
      @errors = @errors.where(errortype: params[:errortype]) if params[:errorType]
      @errors = @errors.where(status: params[:status]) if params[:status]
      @errors = @errors.offset(@offset).limit(@limit)
      @errors.load

      render :errors
    end
  end

  def add
    begin
      AddErrorJob.perform_later(model_params)
    rescue => e
      Rails.logger.error e
    end
    send_machine_response
  end

  def before_set_targets
    models = if filters["sn"]
      Error.where(machine_sn: filters["sn"])
    else
      Error
    end
    models = models.eager_load(:job)
    view_before_set_targets(models, params)
  end

  def before_filter(filter)
    if filter["id"] == "job_number"
      filter["id"] = "jobs.number"
    end
    view_before_filter(filter)
  end

  def before_sort(sort)
    if sort["id"] == "job_number"
      sort["id"] = "jobs.number"
    end
    sort
  end

  private

  # Never trust parameters from the scary internet, only allow the white list through.
  def model_params
    params[:problem] = params[:error] if params[:error].is_a?(ActionController::Parameters)
    error = params.require(:problem)

    job = if error[:job]
            side_num = Sideable.side_letter_to_side_num(error[:lensside])
            FindHelpers::JobFinders.find_active_or_create_by_number_and_side_num(error[:job], side_num)
          else
            nil
          end

    {
      piececounter: error[:piececounter],
      errno: error[:errorno],
      error: error[:error],
      errortype: enum_value(error[:errortype]),
      status: enum_value(error[:status]),
      job: job,
      time: "#{params[:date]} #{params[:time]}",
      machine_sn: params[:sn]
    }
  end
end
