class ExportsController < ApplicationController
  require 'csv'

  include ArchiveHelpers

  def get_jobs
    machine_id = params[:machine_id]
    start_date = params[:start_date]
    end_date = params[:end_date]
    begin
      valid_dates?(start_date, end_date)
      start_date, end_date = date_to_time_converter(start_date), date_to_time_converter(end_date)
      machine = get_machine(machine_id)
      base_headers = ['Job Number', 'Side', 'Date - Time']
      param_headers = {}
      csv_rows = []
      get_models(JobState, :machine_id, machine.id, start_date, end_date)
        .where(state: 'finished')
        .eager_load(:job)
      .find_each do |job_state|
        job = job_state.job
        next if job.nil?
        process_params = job_state.process_params
        p_values = {}
        unless process_params.nil?
          process_params.each do |process_param|
            param_headers[process_param.name] = true
            p_values[process_param.name] = process_param.value
          end
        end
        csv_rows.push({
                        job_values: [job.number, job_state.side, job_state.time],
                        param_values: p_values
        })
      end
      csv_string = CSV.generate do |csv|
        all_headers = [*base_headers, *param_headers.keys]
        csv << all_headers
        csv_rows.each do |row|
          build_row = [*row[:job_values]]
          param_headers.each_key do |header|
            param_values = row[:param_values]
            param_value = param_values.key?(header) ? param_values[header] : ''
            build_row.push(param_value)
          end
          csv << build_row
        end
      end
      render json: { csv: csv_string }, status: :ok
    rescue => e
      render json: { error: e }, status: :unprocessable_entity
    end
  end

  def get_machine_states
    machine_id = params[:machine_id]
    start_date = params[:start_date]
    end_date = params[:end_date]
    begin
      valid_dates?(start_date, end_date)
      start_date, end_date = date_to_time_converter(start_date), date_to_time_converter(end_date)
      machine = get_machine(machine_id)
      headers = ['State', 'Date - Time']
      csv_string = CSV.generate do |csv|
        csv << headers
        get_models(MachineState, :machine_sn, machine.sn, start_date, end_date).find_each do |machine_state|
          csv << [machine_state.state, machine_state.time]
        end
      end
      render json: { csv: csv_string }, status: :ok
    rescue => e
      render json: { error: e }, status: :unprocessable_entity
    end
  end

  def get_job_states
    machine_id = params[:machine_id]
    start_date = params[:start_date]
    end_date = params[:end_date]
    begin
      valid_dates?(start_date, end_date)
      start_date, end_date = date_to_time_converter(start_date), date_to_time_converter(end_date)
      machine = get_machine(machine_id)
      headers = ['Job Number', 'Side', 'State', 'Substate', 'Scrap', 'Scrap Reason', 'Comment', 'Date - Time']
      csv_string = CSV.generate do |csv|
        csv << headers
        get_models(JobState, :machine_id, machine.id, start_date, end_date).eager_load(:job).find_each do |job_state|
          job = job_state.job
          job_number = job.nil? ? 'n/a' : job.number
          if job_state.scrap.nil?
            job_scrap = ''
          else
            job_scrap = job_state.scrap ? 'T' : 'F'
          end
          csv << [job_number, job_state.side, job_state.state, job_state.substate, job_scrap, job_state.scrap_reason, job_state.comment, job_state.time]
        end
      end
      render json: { csv: csv_string }, status: :ok
    rescue => e
      render json: { error: e }, status: :unprocessable_entity
    end
  end

  def get_errors
    machine_id = params[:machine_id]
    start_date = params[:start_date]
    end_date = params[:end_date]
    begin
      valid_dates?(start_date, end_date)
      start_date, end_date = date_to_time_converter(start_date), date_to_time_converter(end_date)
      machine = get_machine(machine_id)
      headers = ['Error', 'Err No.', 'Error Type', 'Status', 'Job Number', 'Date - Time']
      csv_string = CSV.generate do |csv|
        csv << headers
        get_models(Error, :machine_sn, machine.sn, start_date, end_date).eager_load(:job).find_each do |error|
          job = error.job
          job_number = job.nil? ? 'none' : job.number
          csv << [error.error, error.errno, error.errortype, error.status, job_number, error.time]
        end
      end
      render json: { csv: csv_string }, status: :ok
    rescue => e
      render json: { error: e }, status: :unprocessable_entity
    end
  end

  def get_events
    machine_id = params[:machine_id]
    start_date = params[:start_date]
    end_date = params[:end_date]
    begin
      valid_dates?(start_date, end_date)
      start_date, end_date = date_to_time_converter(start_date), date_to_time_converter(end_date)
      machine = get_machine(machine_id)
      headers = ['Message', 'Type', 'Date - Time']
      csv_string = CSV.generate do |csv|
        csv << headers
        get_models(Event, :machine_sn, machine.sn, start_date, end_date).find_each do |event|
          csv << [event.message, event.event_type, event.time]
        end
      end
      render json: { csv: csv_string }, status: :ok
    rescue => e
      render json: { error: e }, status: :unprocessable_entity
    end
  end

  def get_maintenances
    machine_id = params[:machine_id]
    start_date = params[:start_date]
    end_date = params[:end_date]
    begin
      valid_dates?(start_date, end_date)
      start_date, end_date = date_to_time_converter(start_date), date_to_time_converter(end_date)
      machine = get_machine(machine_id)
      headers = ['Maintenance', 'Maintenance Type', 'Status', 'Date - Time']
      csv_string = CSV.generate do |csv|
        csv << headers
        get_models(Maintenance, :machine_sn, machine.sn, start_date, end_date).find_each do |maintenance|
          csv << [maintenance.maintenance, maintenance.maintenance_type, maintenance.status, maintenance.time]
        end
      end
      render json: { csv: csv_string }, status: :ok
    rescue => e
      render json: { error: e }, status: :unprocessable_entity
    end
  end

  def get_calibrations
    machine_id = params[:machine_id]
    start_date = params[:start_date]
    end_date = params[:end_date]
    begin
      valid_dates?(start_date, end_date)
      start_date, end_date = date_to_time_converter(start_date), date_to_time_converter(end_date)
      machine = get_machine(machine_id)
      headers = ['Target', 'Target Name', 'Calibration Type', 'Sub Type', 'Mode', 'User', 'Date - Time']
      csv_string = CSV.generate do |csv|
        csv << headers
        get_models(Calibration, :machine_sn, machine.sn, start_date, end_date).find_each do |calibration|
          csv << [calibration.target, calibration.target_name, calibration.calibration_type, calibration.sub_type, calibration.mode, calibration.user, calibration.time]
        end
      end
      render json: { csv: csv_string }, status: :ok
    rescue => e
      render json: { error: e }, status: :unprocessable_entity
    end
  end

  def get_process_data
    machine_id = params[:machine_id]
    start_date = params[:start_date]
    end_date = params[:end_date]
    begin
      valid_dates?(start_date, end_date)
      start_date, end_date = date_to_time_converter(start_date), date_to_time_converter(end_date)
      machine = get_machine(machine_id)
      headers = ['Job Number', 'Side', 'Date - Time', 'Tag', 'Value']
      csv_string = CSV.generate do |csv|
        csv << headers
        JobProcessParameter
          .joins(:job_process)
          .where(job_process: { created_at: start_date..end_date, machine: machine })
          .eager_load(job_process: :job)
        .find_each do |job_process_parameter|
          job_process = job_process_parameter.job_process
          next if job_process.nil?
          job = job_process.job
          next if job.nil?
          csv << [job.number, job.side, job_process.time, job_process_parameter.name, job_process_parameter.value]
        end
      end
      render json: { csv: csv_string }, status: :ok
    rescue => e
      render json: { error: e }, status: :unprocessable_entity
    end
  end

  private

  def valid_dates?(start_date, end_date)
    if start_date.nil? || end_date.nil?
      raise "invalidDate"
    end

    time_start_date = start_date.to_time
    time_end_date = end_date.to_time

    if time_start_date > time_end_date
      raise "startDateGreaterThanEndDate"
    end
    true
  end

  def date_to_time_converter(date)
    date.to_time
  end

  def get_machine(machine_id)
    machine = Machine.find_by(id: machine_id)
    if machine.nil?
      raise "invalidMachineId"
    end
    machine
  end

  def get_models(model, machine_symbol, machine_value, start_date, end_date)
    shard = if time_exists_in_primary? start_date
              :primary
            else
              :archive
            end

    ActiveRecord::Base.connected_to(shard: shard, role: :reading) do
      model.where(
        model.arel_table[machine_symbol].eq(machine_value).and(
          model.arel_table[:time].gteq(start_date).and(model.arel_table[:time].lteq(end_date))
        )
      )
    end
  end
end
