class EerValueChart < RxModule::Dashboards::DualAxisChart
  include EerDashboard
  include EerValueCalculator
  include MachineConcern
  include MachineDashboard

=begin
  expose("eer_value", "EER Value", "Displays the EER Value over time")
  targets("Machine", "The machine(s) to track", -> { machine_targets }, target_type: "series")
  parameter("number_of_days", "Number of Days", "The number of days to display", :integer, default: 7, min: 1)
  ui_hint(category: 'EER')
  yunit('%')
=end

  def self.machine_targets
    (MachineGroup.all.map do |g|
      next if g.machines.length < 1
      { label: target_to_name(g), value: target_to_id(g), group: "Machine Group", description: "Report EER value for the entire #{g.name} group" }
    end).compact + Machine.all.map do |m|
      { label: target_to_name(m), value: target_to_id(m), group: "Machine", description: "Report EER value for machine #{m.name}, sn #{m.sn}" }
    end
  end

  def combine_mode
    :average
  end

  def build_results!
    number_of_days = @parameters[:number_of_days]
    x_points = get_x_axis(number_of_days)
    all_shifts = get_shifts(x_points.first, x_points.last)
    requested_machines.each do |t|
      machines = [t]
      add_series(target_to_id(t), t.sn, x_points, machines, all_shifts)
    end
    requested_machine_groups.each do |t|
      machines = t.machines.to_a
      add_series(target_to_id(t), t.name, x_points, machines, all_shifts)
    end
  end

  def get_x_axis(number_of_days)
    # to include the current day in the days to display and to still have
    # number_of_days points displayed, remove 1 excess day
    validated_number_of_days = (number_of_days - 1)
    if validated_number_of_days.nil? || validated_number_of_days < 0
      validated_number_of_days = 0
    end
    end_date = Date.today
    start_date = end_date - validated_number_of_days
    range_labels = Tools.create_range_labels(start_date.to_time, end_date.to_time, '1d')
    if number_of_days == 1
      return [range_labels[0]] # return only a single point (start and end label are identical here)
    end
    range_labels
  end

  def add_series(id, label, x_points, machines, all_shifts)
    return if id.nil? || label.nil? || !x_points.is_a?(Array) || !machines.is_a?(Array)
    return if x_points.empty? || machines.empty?
    push_series(id, label)
    x_points.each do |x|
      shifts = all_shifts[x]
      # use the pre-calculated value from UpdateEerValueRoutine, else calculate it for the dashboard
      eer_value_record = EerValue.find_by(date: x, name: label)
      y = eer_value_record.nil? ? calculate_eer_value(machines, shifts, x) : eer_value_record.value
      push_point(id, x, y)
    end
  end
end

