import * as React from "react";
import {observer} from "mobx-react-lite";
import {t} from "@lingui/macro";
import {TabPanelProps} from "../TabPanel"
import {MachineModel} from "../../../stores/MachineStore";
import {MachineDataViewTab} from "../MachineDataViewTab";
import {CalibrationStore} from "../../../stores/CalibrationStore";
import {makeColumn} from "@rx-suite/ui";
import {getDateTimeString} from "../MachineDataView";
import {Moment} from "moment";

type PanelProps = {
    machine: MachineModel;
} & TabPanelProps

export const CalibrationsPanel: React.FunctionComponent<PanelProps> = observer((props) => {
    const {machine} = props;

    const columns = [
        {Header: t`Machine Calibrations`, columns: [
                makeColumn('target', t`Target`, "string"),
                makeColumn('target_name', t`Target Name`, "string"),
                makeColumn('calibration_type', t`Calibration Type`, "string"),
                makeColumn('sub_type', t`Sub Type`, "string"),
                makeColumn('mode', t`Mode`, "string"),
                makeColumn('user', t`User`, "string"),
                {
                    Header: t`Date - Time`,
                    id: 'time',
                    accessor: 'time',
                    Cell: (cellInfo: {value: Moment }) => {
                        return getDateTimeString(cellInfo.value)
                    }
                }
            ]
        }];

    const exportCallback = async (startDate: string, endDate: string) => {

        const calibrations = await machine.exportCSV('calibrations', startDate, endDate);

        return calibrations;
    };

    return (
        <MachineDataViewTab
            columns={columns}
            machine={machine}
            enabledKey={'calibration_data_enabled'}
            tabName={'Calibration'}
            store={CalibrationStore}
            exportedFileName={'Calibrations.csv'}
            exportCallback={exportCallback}
        />
    );
});