import * as React from "react";
import {observer} from "mobx-react-lite";
import {t} from "@lingui/macro";
import {TabPanelProps} from "../TabPanel"
import {MachineModel} from "../../../stores/MachineStore";
import {MachineDataViewTab} from "../MachineDataViewTab";
import {EventModel, EventStore} from "../../../stores/EventStore";
import {makeColumn} from "@rx-suite/ui";
import {getDateTimeString} from "../MachineDataView";
import {Moment} from "moment";

type PanelProps = {
    machine: MachineModel;
} & TabPanelProps

export const EventsPanel: React.FunctionComponent<PanelProps> = observer((props) => {
    const {machine} = props;

    const columns = [
        {
            Header: t`Machine Events`, columns: [
                makeColumn('message', t`Message`, "string"),
                makeColumn('event_type', t`Type`, "string"),
                {
                    Header: t`Date - Time`,
                    id: 'time',
                    accessor: 'time',
                    Cell: (cellInfo: {value: Moment }) => {
                        return getDateTimeString(cellInfo.value);
                    }
                }
            ]
        }];

    const exportCallback = async (startDate: string, endDate: string) => {

        const events = await machine.exportCSV('events', startDate, endDate);

        return events;
    };

    return (
        <MachineDataViewTab
            columns={columns}
            machine={machine}
            enabledKey={'event_data_enabled'}
            tabName={'Event'}
            store={EventStore}
            exportedFileName={'Events.csv'}
            exportCallback={exportCallback}
        />
    );
});
