import * as React from "react";
import {observer} from "mobx-react-lite";
import {t} from "@lingui/macro";
import {TabPanelProps} from "../TabPanel"
import {MachineModel} from "../../../stores/MachineStore";
import {MachineDataViewTab} from "../MachineDataViewTab";
import {MaintenanceModel, MaintenanceStore} from "../../../stores/MaintenanceStore";
import {makeColumn, SelectField} from "@rx-suite/ui";
import {Filter, ReactTableFunction} from "react-table";
import {getDateTimeString} from "../MachineDataView";
import {Moment} from "moment";

type PanelProps = {
    machine: MachineModel;
} & TabPanelProps

export const MaintenancePanel: React.FunctionComponent<PanelProps> = observer((props) => {
    const {machine} = props;

    const columns = [
            {
                Header: t`Machine Maintenance`,
                columns: [
                    makeColumn('maintenance', t`Maintenance`, 'string'),
                    {
                        Header: t`Maintenance Type`,
                        id: 'maintenance_type',
                        accessor: 'maintenance_type',
                        filterable: true,
                        Filter: ({ filter, onChange }: { filter: Filter, onChange: ReactTableFunction}) => (
                            <SelectField
                                options={
                                    [
                                        {label: t`unknown`, value: 0},
                                        {label: t`counter`, value: 1},
                                        {label: t`date`, value: 2},
                                        {label: t`time period`, value: 3},
                                        {label: t`event`, value: 4},
                                        {label: t`all`, value: ""}
                                    ]
                                }
                                onChange={(name, value) => onChange(value)}
                                value={filter ? filter.value : ""}
                            />
                        )
                    },
                    {
                        Header: t`Status`,
                        id: 'status',
                        accessor: 'status',
                        filterable: true,
                        Filter: ({ filter, onChange }: { filter: Filter, onChange: ReactTableFunction}) => (
                            <SelectField
                                options={
                                    [
                                        {label: t`raised`, value: 0},
                                        {label: t`acknowledged`, value: 1},
                                        {label: t`finished`, value: 2},
                                        {label: t`all`, value: ""}
                                    ]
                                }
                                onChange={(name, value) => onChange(value)}
                                value={filter ? filter.value : ""}
                            />
                        )
                    },
                    {
                    Header: t`Date - Time`,
                    id: 'time',
                    accessor: 'time',
                    Cell: (cellInfo: {value: Moment }) => {
                        return getDateTimeString(cellInfo.value);
                    }
                }
                ]
            }
        ];

    const exportCallback = async (startDate: string, endDate: string) => {

        const maintenances = await machine.exportCSV('maintenances', startDate, endDate);

        return maintenances;
    };

    return (
        <MachineDataViewTab
            machine={machine}
            columns={columns}
            enabledKey={'maintenance_enabled'}
            tabName={'Maintenance'}
            store={MaintenanceStore}
            exportedFileName={'Maintenances.csv'}
            exportCallback={exportCallback}
        />
    );
});