import * as React from "react";
import {observer} from "mobx-react-lite";
import {t} from "@lingui/macro";
import {TabPanelProps} from "../TabPanel"
import {MachineModel} from "../../../stores/MachineStore";
import {MachineDataViewTab} from "../MachineDataViewTab";
import {makeColumn, makeFormattedDateTimeColumn} from "@rx-suite/ui";
import {JobProcessParameterStore} from "../../../stores/JobProcessParameterStore";
import {useJobSideColumn} from "../../../hooks/table_columns/useJobSideColumn";

type PanelProps = {
    machine: MachineModel;
} & TabPanelProps

export const ProcessDataPanel: React.FunctionComponent<PanelProps> = observer((props) => {
    const {machine} = props;
    const store = JobProcessParameterStore;

    const columns = [
        {
            Header: t`Process Data`,
            columns: [
                makeColumn('job_process.job.number', 'Job Number', 'string'),
                useJobSideColumn(machine, `job_process.side`),
                makeFormattedDateTimeColumn('job_process.time', 'Date - Time', 'dateTime'),
                makeColumn('name', 'Tag', 'string'),
                makeColumn('value', 'Value', 'string')
            ]
        }
    ];

    const exportCallback = async (startDate: string, endDate: string) => {
        const processData = await machine.exportCSV('process_data', startDate, endDate);
        return processData;
    };

    return (
        <MachineDataViewTab
            columns={columns}
            machine={machine}
            enabledKey={'process_data_enabled'}
            tabName={'Process'}
            store={store}
            defaultSorted={[{id: "job_process.time", desc: true}]}
            exportedFileName={'ProcessData.csv'}
            exportCallback={exportCallback}
        />
    );
});


