import * as React from "react";
import {observer} from "mobx-react-lite";
import {BarChart, ReportComponentMetaData, TemplateReportComponent} from "@rx-suite/ui";

type MachineAbortReportProps = {
    component: {
        meta: ReportComponentMetaData;
        data: any;
    };
};

export const MachineAbortReport: React.FunctionComponent<MachineAbortReportProps> = observer(({component}) => {
    const data = component.data;

    return (
        <TemplateReportComponent meta={component.meta}>
            <BarChart
                series={data.series}
                options={
                    {
                        title: {
                            text: 'Top Machine Aborts',
                            align: 'center'
                        },
                        chart: {
                            toolbar: {
                                show: false
                            }
                        },
                        legend: {
                            show: false
                        },
                        dataLabels: {
                            formatter: (value: number) => (value.toFixed(0)) + "%"
                        },
                        tooltip: {
                            y: {
                                formatter: (value: number) => (value.toFixed(0)) + "%"
                            }
                        },
                        plotOptions: {
                            bar: {
                                barHeight: '60%'
                            }
                        },
                        xaxis: {
                            categories: data.categories,
                            labels: {
                                formatter: (value: number) => (value.toFixed(0)) + "%"
                            },
                            tickAmount: 6,
                        },
                        yaxis: {
                            labels: {
                                maxWidth: 400,
                            },
                            // If less than 100, round up to nearest 10s
                            max: (max: number) => max < 100 ? Math.ceil(max / 20) * 20 : 100,
                            tickAmount: 6,
                            forceNiceScale: true
                        },
                        colors: data.colors ? data.colors : ['#FF0000']
                    }
                }
                height={'100%'}
                horizontal/>
        </TemplateReportComponent>
    );
})