import * as React from "react";
import {observer} from "mobx-react-lite";
import {BarChart, ReportComponentMetaData, TemplateReportComponent} from "@rx-suite/ui";
import {useMemo} from "react";

type MachineErrorReportProps = {
    component: {
        meta: ReportComponentMetaData;
        data: any;
    };
};

export const MachineErrorReport: React.FunctionComponent<MachineErrorReportProps> = observer(({component}) => {
    const data = component.data;

    const multiLineCategories = useMemo(() => {
        if (!data || !data.categories) {
            return [];
        }
        return data.categories.map((category: string) =>
            category.replace(/.{35}\S*\s+/, "$&#split#").split(/\s+#split#/)
        );
    }, [data, data.categories]);

    return (
        <TemplateReportComponent meta={component.meta}>
            <BarChart
                series={data.series}
                options={
                    {
                        title: {
                            text: 'Top Machine Errors',
                            align: 'center'
                        },
                        chart: {
                            toolbar: {
                                show: false
                            }
                        },
                        legend: {
                            show: false
                        },
                        dataLabels: {
                            formatter: (value: number) => (value.toFixed(0)) + "%"
                        },
                        tooltip: {
                            y: {
                                formatter: (value: number) => (value.toFixed(0)) + "%"
                            }
                        },
                        plotOptions: {
                            bar: {
                                barHeight: '60%'
                            }
                        },
                        xaxis: {
                            categories: multiLineCategories,
                            labels: {
                                formatter: (value: number) => (value.toFixed(0)) + "%"
                            },
                            tickAmount: 6
                        },
                        yaxis: {
                            labels: {
                                maxWidth: 400,
                            },
                            // If less than 100, round up to nearest 10s
                            max: (max: number) => max < 100 ? Math.ceil(max / 20) * 20 : 100,
                            tickAmount: 6,
                            decimalsInFloat: 1,
                            forceNiceScale: true
                        },
                        colors: ['#FF0000']
                    }
                }
                height={'100%'}
                horizontal/>
        </TemplateReportComponent>
    );
})