import * as React from 'react';
import {t, Trans} from "@lingui/macro";
import {green, red} from "@material-ui/core/colors";
import Grid from "@material-ui/core/Grid";
import {Button, Divider, makeStyles, Typography} from "@material-ui/core";
import {observer} from "mobx-react-lite";

type ServiceControllerViewProps = {
    name?: string;
    online: boolean;
    enabled: boolean;
    startAction: () => void;
    stopAction: () => void;
    startText?: string;
    stopText?: string;
}

const useStyles = makeStyles((theme) => ({
    status: {
        marginLeft: '2px',
        fontWeight: 'bold',
        color: (online) => online ? green[600] : red[600]
    },
    divider: {
        margin: theme.spacing(2),
    },
}));

export const ServiceController: React.FunctionComponent<ServiceControllerViewProps> = observer((props) => {
    const {online, startAction, stopAction, startText, stopText, enabled, name} = props;
    const classes = useStyles(online);

    if(!enabled) return null;

    let status = t`Offline`;
    let handleClick = startAction;
    let actionText = startText ? t`${startText}` : t`Start Server`;

    if(online) {
        status = t`Online`;
        handleClick = stopAction;
        actionText = stopText ? t`${stopText}` : t`Stop Server`;
    }

    return(
        <div>
            {name && <Typography variant={'h5'}>{name}</Typography>}
            <Grid container>
                <Grid item>
                    <Typography variant={'h6'}>
                        <Trans>Status</Trans>:
                    </Typography>
                </Grid>
                <Grid item>
                    <Typography variant={'h6'} className={classes.status}>{status}</Typography>
                </Grid>
            </Grid>
            <Button size="medium" variant="contained" color="secondary" onClick={handleClick}>{actionText}</Button>
            <Divider className={classes.divider}/>
        </div>
    )
});
