import * as React from "react";
import { t } from "@lingui/macro"
import {MachineModel} from "../../stores/MachineStore";
import {Filter, ReactTableFunction} from "react-table";
import {SelectField} from "@rx-suite/ui";
import {useEffect, useMemo} from "react";
import {MachineTypeStore} from "../../stores/MachineTypeStore";

export const useJobSideColumn = (machine: MachineModel, sideAccessor: string) => {

    const [isBoxCoater, setIsBoxCoater] = React.useState<boolean>(false);

    useEffect(() => {
        (async() => {
            if(!machine.machine_type_id){
                return false;
            }

            const type = await MachineTypeStore.get(machine.machine_type_id)

            if(!type){
                setIsBoxCoater(false)
            }else{
                setIsBoxCoater(type.isBoxCoater)
            }
        })();
    }, [machine]);

    const filterOptions = useMemo(() => {
        if (isBoxCoater) {
            return [
                {label: t`concave`, value: 0},
                {label: t`convex`, value: 1},
                {label: t`both`, value: ""}
            ];
        }

        return [
            {label: t`left`, value: 0},
            {label: t`right`, value: 1},
            {label: t`both`, value: ""}
        ];
    }, [isBoxCoater]);

    return (
        {
            Header: t`Side`,
            id: sideAccessor, // id is used when filtering to go through the db, so the accessor needs to be here as well
            filterable: true,
            accessor: sideAccessor,
            Cell: (cellInfo: { value: string }) => {

                if (!cellInfo || cellInfo.value == null) {
                    return null;
                }

                const side = cellInfo.value;

                return (
                    <div>{side}</div>
                );
            },
            Filter: ({ filter, onChange }: { filter: Filter, onChange: ReactTableFunction }) => (
                <SelectField
                    options={filterOptions}
                    onChange={(name, value) => onChange(value)}
                    value={filter ? filter.value : ""}
                />
            )
        }
    );
}

