import * as React from 'react';
import {DataArchiveSettingStore} from "../stores/DataArchiveSettingStore";
import {observer} from "mobx-react-lite";
import {LinearProgress, makeStyles, Typography} from "@material-ui/core";
import {ModelForm, ModelInput} from "@rx-suite/ui";
import {t} from "@lingui/macro";

const useStyles = makeStyles((theme) => ({
    expiryTime: {
        marginTop: '0.5rem'
    },
    cleanUpRoutine: {
        marginTop: '0.5rem'
    }
}));

export const DataArchiveSettings = observer(() => {

    const classes = useStyles();

    const settings = DataArchiveSettingStore.selectedModel;

    if(!settings) return <LinearProgress />;

    const standardInputProps = { inputProps: { min: 0 } };

    return (
        <ModelForm model={settings} suppressMakeCopy suppressImportExport>
            <div className={classes.expiryTime}>
                <Typography variant={"h5"}>
                    {t`Data Expiry Time`}
                </Typography>
                <ModelInput name={"job_max_age"} type="number" InputProps={{ inputProps: { min: 0, max: 8760 } }} />
            </div>
            <div className={classes.cleanUpRoutine}>
                <Typography variant={"h5"}>
                    {t`Data Clean Up Routine`}
                </Typography>
                <ModelInput name={"cycle_interval"} type="number" InputProps={standardInputProps} />
                <ModelInput name={"delete_count_order"} type="number" InputProps={standardInputProps} />
                <ModelInput name={"delete_count_job"} type="number" InputProps={standardInputProps} />
                <ModelInput name={"delete_count_machine_state"} type="number" InputProps={standardInputProps} />
                <ModelInput name={"delete_count_error"} type="number" InputProps={standardInputProps} />
                <ModelInput name={"delete_count_maintenance"} type="number" InputProps={standardInputProps} />
                <ModelInput name={"delete_count_event"} type="number" InputProps={standardInputProps} />
                <ModelInput name={"delete_count_calibration"} type="number" InputProps={standardInputProps} />
                <ModelInput name={"delete_count_kpi_cache"} type="number" InputProps={standardInputProps} />
            </div>
        </ModelForm>
    );
});
