import * as React from 'react';
import {observer} from "mobx-react-lite";
import {FormProps, ModelForm, ModelInput} from "@rx-suite/ui";
import {t} from '@lingui/macro';
import {BlockPickingModel} from "../../stores/BlockPickingStore";
import {BPConnectionSettingsTab} from "../../components/block_picking/settings_tabs/BPConnectionSettingsTab";
import {BPDataSettingsTab} from "../../components/block_picking/settings_tabs/BPDataSettingsTab";
import {BPDisplaySettingsTab} from "../../components/block_picking/settings_tabs/BPDisplaySettingsTab";
import {TabsContainer} from "../../components/generic/TabsContainer";
import {BPTagsSettingsTab} from "../../components/block_picking/settings_tabs/BPTagsSettings";
import {WorkstationNameField} from "../../components/generic/WorkstationNameField";

export const BlockPickingForm: React.FunctionComponent<FormProps> = observer(({model}) => {

    return (
        <ModelForm model={model} suppressImportExport={true} suppressMakeCopy={true}>
            <WorkstationNameField name={'name'}/>
            <TabsContainer
                tabs={[{
                    label: t`Connection`,
                    component: <BPConnectionSettingsTab model={model as BlockPickingModel}/>,
                    disabled: false,
                    defaultTab: true
                },
                {
                    label: t`Data`,
                    component: <BPDataSettingsTab model={model as BlockPickingModel}/>,
                    disabled: false,
                    defaultTab: true
                },
                {
                    label: t`Display`,
                    component: <BPDisplaySettingsTab model={model as BlockPickingModel}/>,
                    disabled: false,
                    defaultTab: true
                },
                {
                    label: t`Custom Tags`,
                    component: <BPTagsSettingsTab model={model as BlockPickingModel}/>,
                    disabled: false,
                    defaultTab: true
                }]}
                variant="scrollable"
            />
        </ModelForm>
    );
})