import * as React from 'react';
import {useSnackbar} from "notistack";
import {Grid, makeStyles} from "@material-ui/core";
import {BlockPickingModel} from "../../stores/BlockPickingStore";
import {Trans} from "@lingui/macro";
import {observer} from "mobx-react-lite";
import {OperatorSearchField} from "../../components/generic/OperatorSearchField";
import {WarningBox} from "../../components/generic/WarningBox";
import {BlockInfo, BlockInfoPanel} from "../../components/block_picking/BlockInfoPanel";
import {BlockPickedCountCard} from "../../components/block_picking/BlockPickedCountCard";
import {t} from '@lingui/macro';
import {sendEnqueueSnackbar, WorkstationHelper, WorkstationGenericWrapper, Conditional} from "@rx-suite/ui";
import {Alert} from "@material-ui/lab";
import Box from "@material-ui/core/Box";
import {DcsConnectionLabel} from "../../components/block_picking/DcsConnectionLabel";


type BlockPickingWorkstationProps = {
    model: BlockPickingModel
};

type errorModel = {
    error: string;
    details? : string;
};

const useStyles = makeStyles((theme) => ({
    searchBoxFull: {
        width: '100%'
    },
    searchBoxWithCount: {
        width: '80%'
    },

}));

export const BlockPickingWorkstation: React.FunctionComponent<BlockPickingWorkstationProps> = observer(({model}) => {
    const {enqueueSnackbar} = useSnackbar();
    const classes = useStyles();

    const [displayModel, setDisplayModel] = React.useState<undefined|BlockInfo>(undefined);
    const [errorModel, setErrorModel] = React.useState<undefined|errorModel>(undefined);

    const search = async (searchVal: string) => {
        setDisplayModel(undefined);
        setErrorModel(undefined);
        const resp = await model.getBlockData(searchVal);
        const content = await resp.json();
        if (!resp.ok) {
            if (content.details) {
                console.error(content.details);
            }
            sendEnqueueSnackbar(t`Failed to find block data.`, {variant: "error"}, enqueueSnackbar);
            return setErrorModel(content);
        }
        setDisplayModel(content);
        sendEnqueueSnackbar(t`Block data found.`, {variant: "success"}, enqueueSnackbar);
    }
    
    return (
        <WorkstationHelper stores={[]}>
            <WorkstationGenericWrapper>
                <Conditional condition={model.isUsingLegacyConnection}>
                    <Box mb={1}>
                        <Alert severity='warning' mb={1}>
                            <Trans>This workstation is using a legacy DCS connection. Please contact support to have it updated.</Trans>
                        </Alert>
                    </Box>
                </Conditional>
            </WorkstationGenericWrapper>
            <Grid container>
                <Grid item className={model.hide_block_count ? classes.searchBoxFull : classes.searchBoxWithCount}>
                    <OperatorSearchField searchFieldPlaceHolderText={t`Search Block Piece Information`} searchFunction={search}/>
                    <Grid container justifyContent='flex-end'>
                        <DcsConnectionLabel connectionId={model.external_dcs_id}/>
                    </Grid>
                </Grid>
                <BlockPickedCountCard model={model}/>
            </Grid>
            {errorModel &&
                <WorkstationGenericWrapper><
                    WarningBox text={errorModel.error}/>
                </WorkstationGenericWrapper>
            }
            <WorkstationGenericWrapper>
                <BlockInfoPanel model={displayModel} hideTrayNumber={model.hide_tray_number} hideJobNumber={model.hide_job_number}/>
            </WorkstationGenericWrapper>
        </WorkstationHelper>
    );
});
