import * as React from 'react';
import {BreadModel, makeColumn, ModelTableUpdateButton, SelectField} from "@rx-suite/ui";
import {t} from "@lingui/macro";
import {observer} from "mobx-react-lite";
import {ErrorFilterModel, ErrorFilterStore} from "../../../stores/ErrorFilterStore";
import {ErrorFilterForm} from "./ErrorFilterForm";
import {MachineGroupStore} from "../../../stores/MachineGroupStore";
import {ToggleFilterSwitch} from "../../../components/generic/ToggleFilterSwitch";

export const ErrorCustomFilterBreadModel = observer(() => {
    const columns = React.useMemo(() => {
        return [
            {
                Header: t`Target`,
                accessor: d => d,
                id: 'group',
                Cell: (cellInfo: {value: ErrorFilterModel }) => {
                    const group = MachineGroupStore.models.find(g => g.getId() == cellInfo.value.filter_target_id);

                    if(!group){
                        return null;
                    }

                    return group.name;
                },
            },
            makeColumn('name', t`Rule Name`, "string"),
            makeColumn('error_codes', t`Error Codes`, "string"),
            makeColumn('behaviour', t`Behaviour`, "string"),
            {
                Header: t`Enabled`,
                accessor: d => d,
                id: 'enabled',
                Cell: (cellInfo: {value: ErrorFilterModel }) => {
                    return <ToggleFilterSwitch model={cellInfo.value}/>
                }
            }
        ]
    }, [MachineGroupStore.models]);

    return (
        <BreadModel
            store={ErrorFilterStore}
            Form={ErrorFilterForm}
            columns={columns}
            tableProps={
                {
                    replacementActions: {
                        Edit: (cell) => <ModelTableUpdateButton store={cell.store} model={cell.model} />,
                        Delete: undefined
                    },
                    defaultFiltered: [{id: 'system_default', value: [false]}],
                    suppressWrapper: true,
                    noLinkColumns: ['enabled']
                }
            }
        />
    )
});