import * as React from 'react';
import {FormProps, HintButton, ModelForm, ModelInput, ModelSelect} from "@rx-suite/ui";
import {observer} from "mobx-react-lite";
import {MachineStore} from "../../stores/MachineStore";
import {t} from "@lingui/macro";
import {Grid, IconButton, makeStyles} from "@material-ui/core";
import CloseIcon from "@material-ui/icons/Close";

const useStyles = makeStyles(theme => ({
    colorFieldWrap: {
        flexGrow: 1
    },
    hintFieldWrap: {
        marginTop: '10px',
    }
}));

export const MachineGroupsForm: React.FunctionComponent<FormProps> = observer(({model}) => {
    const classes = useStyles();

  const options = MachineStore.getAll().sort((a,b) => a.extendedName.localeCompare(b.extendedName)).map(m => {
    return {
      value: m.getId(),
      label: m.extendedName
    }
  })

  return (
    <ModelForm model={model} suppressMakeCopy suppressImportExport>
        <ModelInput name={"name"}/>
        <Grid container>
            <ModelInput
                className={classes.colorFieldWrap}
                label={"Group Color"}
                name={"color"}
                type={"color"}
            />
            <div>
                <Grid item container alignItems={'center'}  className={classes.hintFieldWrap}>
                    {model.viewModel.color &&
                        <IconButton onClick={()=>{ model.viewModel.color = null }}>
                            <CloseIcon color={'error'}/>
                        </IconButton>
                    }
                    <HintButton title={t`Group Color`} text={t`This color will be applied everywhere throughout the application, including dashboards and reports.`}/>
                </Grid>
            </div>
        </Grid>
        <ModelSelect
            multiple
            loading={MachineStore.isLoading}
            name={"machine_ids"}
            options={options}
        />
    </ModelForm>
  );
});
