import * as React from 'react';
import {observer} from "mobx-react-lite";
import {FormProps, ModelForm, ModelInput} from "@rx-suite/ui";
import {t, Trans} from "@lingui/macro";
import { Alert } from '@material-ui/lab';
import {TabsContainer} from "../../../components/generic/TabsContainer";
import {MachineUsageSettingsTab} from "../../../components/machine/settings_form/MachineUsageSettingsTab";
import {MachineModel} from "../../../stores/MachineStore";
import {MachineGeneralSettingsTab} from "../../../components/machine/settings_form/MachineGeneralSettingsTab";

export const MachineForm:React.FunctionComponent<FormProps> = observer(({ model }) => {

  return (
      <ModelForm model={model} maxWidth={'xl'} suppressMakeCopy suppressImportExport>
          {model.archived && <Alert severity="warning"><Trans>This machine is archived and will not appear in other modules, reports, and dashboard components.</Trans></Alert>}
          <ModelInput name={"sn"} disabled={!model.isNew}/>
          <TabsContainer
              variant="scrollable"
              scrollButtons="auto"
              tabs={[
                  {
                      label: t`General`,
                      component: <MachineGeneralSettingsTab/>,
                      defaultTab: true
                  },
                  {
                      label: t`Usage`,
                      component: <MachineUsageSettingsTab machine={model as MachineModel}/>,
                  }
              ]}
          />
      </ModelForm>
  )
});
