import * as React from 'react';
import { Model, Transport } from '@rx-suite/data';
import {observer} from "mobx-react-lite";

import {t} from "@lingui/macro";
import {ModelForm, ModelInput, ModelSelect} from "@rx-suite/ui";

type PushTargetFormProps = {
  model: Model
}

type App = {
  module_name: string,
  module_id: string
}

export const PushTargetForm: React.FunctionComponent<PushTargetFormProps> = observer(({ model }) => {
  const [ apps, setApps ] = React.useState([]);
  const [ loading, setLoading ] = React.useState(true);
  React.useEffect(() => {
    const tmp = async () => {
      setLoading(true);
      const response = await Transport.fetch('/api/rx_module/apps');
      const json = await response.json();
      const options = json.map((j: App ) => ({
        value: j.module_id,
        label: j.module_name
      }));
      setApps(options);
      setLoading(false);
    };
    tmp();
  }, []);
  return(
    <ModelForm model={model}>
      <ModelSelect
        options={apps}
        name={"app"}
        loading={loading}
      />
      <ModelInput name={"machine"} />
      <ModelInput name={"machine_state"} />
      <ModelInput name={"job_state"} />
      <ModelInput name={"error"} />
      <ModelInput name={"maintenance"} />
      <ModelInput name={"event"} />
      <ModelInput name={"job_process_parameter"} />
      <ModelInput name={"job"} />
      <ModelInput name={"order"} />
      <ModelInput name={"block"} />
      <ModelInput name={"calibration"} />
      <ModelInput name={"box_coater_event"} />
    </ModelForm>
  )
});
