import {attribute, DataStore, Model, model} from '@rx-suite/data';
import * as yup from 'yup';
import {t} from "@lingui/macro";
import {computed} from "mobx";

@model({
    transportLayer: "/api/block_pickings",
    permissionTarget: "BlockPicking",
    routingInfo: {
        create: '/settings/block_pickings/create',
        read: '/settings/block_pickings/:id/edit',
        update: '/settings/block_pickings/:id/edit'
    }
})
export class BlockPickingModel extends Model {
    @attribute({
        label: t`Name`,
        validation: yup.string().required(t`Name is required`)
    })
    name: string = '';

    @attribute({
        label: t`Hide Tray Number`,
        validation: yup.boolean()
    })
    hide_tray_number: boolean = false;

    @attribute({
        label: t`Hide Job Number`,
        validation: yup.boolean()
    })
    hide_job_number: boolean = false;

    @attribute({
        label: t`Hide Block Count`,
        validation: yup.boolean()
    })
    hide_block_count: boolean = false;

    @attribute({
        label: t`Hide Block Count`,
        validation: yup.boolean()
    })
    use_scan_count: boolean = false;

    @attribute({
        label: t`Send Job States`,
        validation: yup.boolean()
    })
    send_job_states: boolean = false;

    @attribute({
        label: t`Block Count`,
        validation: yup.number()
    })
    block_count: number = 0;

    @attribute({
        label: t`Shift Ids`,
        validation: yup.array()
    })
    shift_reference_ids: number[] = [];

    @attribute({
        label: t`active shift id`,
        validation: yup.number().nullable()
    })
    active_shift_id: number | null = null;

    @attribute({
        label: t`Hub Ids`,
        validation: yup.array()
    })
    hub_shift_ids: number[] = [];

    @attribute({
        label: t`LMS Connection`,
        validation: yup.number().typeError(t`Select a valid LMS connection.`)
    })
    external_dcs_id: number | null = null;

    // LEGACY ATTRIBUTES: DO NOT ALLOW EDIT - ONLY KEEP FOR VIEW PURPOSES //

    @computed
    get isUsingLegacyConnection(){
        return this.external_dcs_id == null;
    }

    @attribute({
        label: t`LMS IP Address`,
        validation: yup.string().nullable()
    })
    ip_address: string | null = null;

    @attribute({
        label: t`Port`,
        validation: yup.number().nullable()
    })
    port: number | null = null;

    @attribute({
        label: t`Device Type`,
        validation: yup.string().nullable()
    })
    device_type: string | null = null;

    @attribute({
        label: t`Hold connection open?`,
        validation: yup.boolean().nullable()
    })
    hold_connection: boolean = false;

    @attribute({
        label: t`Delay on socket open`,
        validation: yup.number().nullable()
    })
    delay_on_socket_open: number | null = null;

    // END OF LEGACY ATTRIBUTES //

    async getBlockData(jobNumber: string) {

        return await fetch(`/api/block_pickings/${this.id}/search`, {
            method: "POST",
            headers: {'Content-Type': 'application/json'},
            body: JSON.stringify({tray_number: jobNumber})
        });
    }

    constructor() {
        super();
        super.setup();
    }
}

export const BlockPickingStore = BlockPickingModel.store as DataStore<BlockPickingModel>;
