import {attribute, DataStore, Model, model} from '@rx-suite/data';
import * as yup from 'yup';
import {t} from "@lingui/macro";

@model({
    transportLayer: "/api/block_picking_tags",
    permissionTarget: "BlockPicking",
    routingInfo: {
        create: '/settings/block_picking_tags/create',
        read: '/settings/block_picking_tags/:id/edit',
        update: '/settings/block_picking_tags/:id/edit'
    }
})
export class BlockPickingTagModel extends Model {
    @attribute({
        label: t`Name`,
        validation: yup.string().required(t`Name is required`)
    })
    name: string = '';

    @attribute({
        label: t`Lms Tag`,
        validation: yup.string().required(t`Tag is required`)
    })
    tag: string = '';

    @attribute({
        label: t`Regex (Optional)`,
        validation: yup.string().nullable()
    })
    regex: string | null = null;

    @attribute({
        label: t`Block Picking id`,
        validation: yup.number()
    })
    block_picking_id: number = 0;

    constructor() {
        super();
        super.setup();
    }
}

export const BlockPickingTagStore = BlockPickingTagModel.store as DataStore<BlockPickingTagModel>;
