import {attribute, DataStore, model, Model} from '@rx-suite/data';

import {t} from "@lingui/macro";
import * as yup from "yup";

@model({
  transportLayer: "/api/box_coater_recipes",
  permissionTarget: 'BoxCoaterRecipe',
  routingInfo: {
    create: "/settings/box_coater_recipes/create",
    update: "/settings/box_coater_recipes/:id/edit",
    read: "/settings/box_coater_recipes/:id"
  }
})
export class BoxCoaterRecipeModel extends Model {
  @attribute({
    label: t`Display Name`,
    validation: yup.string().required(t`Recipe Code is required`).min(3)
  })
  name: string = "";

  @attribute({
    label: t`Recipe Code`,
    validation: yup.string().required(t`Recipe Code is required`).min(3)
  })
  recipe_code: string = "";

  @attribute({label: t`Processing Time (in seconds)`, validation: yup.number().positive().integer().min(30)})
  processing_time: number = 300;

  @attribute({ label: t`Dome Side`})
  dome_side: DomeSide = "CC";

  constructor() {
    super();
    super.setup();
  }
}

type DomeSide = 'CC' | 'CX' | 'both'

export const BoxCoaterRecipeStore = BoxCoaterRecipeModel.store as DataStore<BoxCoaterRecipeModel>;
