import {attribute, DataStore, model, Model} from '@rx-suite/data';

import {t} from "@lingui/macro";
import * as yup from "yup";
import {override} from "mobx";
import {JobStore} from "./JobStore";

@model({
  transportLayer: "/api/errors",
  routingInfo: { }
})
export class ErrorModel extends Model {
  @attribute({ label: t`Piece Counter`, validation: yup.number() })
  piececounter: number = 0;

  @attribute({ label: t`Err. No`, validation: yup.number()})
  errno: number = 0;

  @attribute({ label: t`Error`, validation: yup.string() })
  error: string = "";

  @attribute({ label: t`Error Type`, validation: yup.number() })
  errortype: number = 0;

  @attribute({ label: t`Status`, validation: yup.number() })
  status: number = 0;

  @attribute({label: t`Job`, connects: "JobModel"})
  job_id: string | null = null;

  @attribute({ label: t`Time`, validation: yup.date().required() })
  time: Date = new Date();

  @attribute({ label: t`Machine sn`, validation: yup.string().required()})
  machine_sn: string = "";

  @attribute({ label: t`Created At`, autoForm: false })
  created_at: Date = new Date();

  @attribute({ label: t`Updated At`, autoForm: false })
  updated_at: Date = new Date();

  static types = {
    time: "date"
  }

  @override
  updateFromJson(json: Model, isClone?: boolean) {
    super.updateFromJson(json, isClone);
    if(json.job_id){
      let job = JobStore.data.get(json.job_id.toString());
      if(!job){
        job = JobStore.createModel(json.job);
        JobStore.setModel(job)
      }
      this.job_id = job.getId()
    }
  }

  constructor() {
    super();
    super.setup();
  }
}

export const ErrorStore = ErrorModel.store as DataStore<ErrorModel>;
