import {attribute, DataStore, model, Model} from "@rx-suite/data";

import {t} from "@lingui/macro";
import * as yup from 'yup';
import {override} from "mobx";
import {JobStore} from "./JobStore";

export type JobStateName = 'started' | 'aborted' | 'finished' | 'postponed' | 'skipped' | 'restarted';

@model({
  transportLayer: "/api/job_states",
  routingInfo: {}
})
export class JobStateModel extends Model {
  @attribute({label: t`Job`, connects: "JobModel"})
  job_id: string | null = null;

  @attribute({ label: t`Batch Id`, validation: yup.string().nullable()})
  batch_id: string  = "";

  @attribute({ label: t`Process`, validation: yup.string().required() })
  process: string  = "";

  @attribute({ label: t`Side`, validation: yup.string().required() })
  side: string  = "";

  @attribute({ label: t`State`, validation: yup.string().required() })
  state: string  = "";

  @attribute({ label: t`Comment`, validation: yup.string().nullable() })
  comment: string  = "";

  @attribute({ label: t`Scrap`, validation: yup.boolean().nullable() })
  scrap: boolean  = false;

  @attribute({ label: t`Scrap Reason`, validation: yup.boolean().nullable() })
  scrap_reason: string  = "";

  @attribute({ label: t`Time`, validation: yup.date().required() })
  time: Date = new Date();

  @attribute({ label: t`Machine Id`, validation: yup.string().required() })
  machine_id: string  = "";

  @attribute({ label: t`Substate`, validation: yup.string().nullable() })
  substate: string  = "";

  @attribute({ label: t`Created At`, autoForm: false })
  created_at: Date = new Date();

  @attribute({ label: t`Updated At`, autoForm: false })
  updated_at: Date = new Date();

  static types = {
    time: "date"
  }

  @override
  updateFromJson(json: Model, isClone?: boolean) {
    super.updateFromJson(json, isClone);
    let job = JobStore.data.get(json.job.id.toString());
    if(!job){
      job = JobStore.createModel(json.job);
      JobStore.setModel(job)
    }
    this.job_id = job.getId()
  }

  async getProcessParams() {

    const resp = await fetch(`/api/get_process_params`, {
      method: "POST",
      headers: {
        "Content-Type": "application/json"
      },
      body: JSON.stringify({ job_state_id: this.id })
    });

    const processParams = await resp.json();

    return processParams;
  }

  constructor() {
    super();
    super.setup();
  }
}

export const JobStateStore = JobStateModel.store as DataStore<JobStateModel>;
