import {attribute, DataStore, model, Model} from '@rx-suite/data';
import {t} from "@lingui/macro";
import * as yup from "yup";

@model({
  transportLayer: "/api/production_settings",
  permissionTarget: 'ProductionSetting',
  routingInfo: {
    update: "/settings/production_settings/:id/edit",
    read: "/settings/production_settings/:id"
  },
  singleton: true
})
export class ProductionSettingModel extends Model {
  @attribute({ label: t`Throughput Metric`})
  throughput_metric: 'lens' | 'pairs' = 'lens';

  @attribute({
    label: t`OEE Target (%)`,
    validation: yup.number().min(1).max(100).required()})
  machine_oee_target: number = 70;

  @attribute({
    label: t`Utilization Target (%)`,
    validation: yup.number().min(1).max(100).required()})
  machine_utilization_target: number = 70;

  @attribute({
    label: t`Uptime Target (%)`,
    validation: yup.number().min(1).max(100).required()})
  machine_uptime_target: number = 80;

  @attribute({
    label: t`Use simplified machine states`,
    validation: yup.boolean().required()})
  simplified_machine_states: boolean = false ;

  @attribute({ label: t`Record Box Coater Performance Metrics`})
  is_eer_lab: boolean = false;

  constructor() {
    super();
    super.setup();
  }
}

export const ProductionSettingStore = ProductionSettingModel.store as DataStore<ProductionSettingModel>;
