import {attribute, DataStore, model, Model} from '@rx-suite/data';

import {t} from "@lingui/macro";
import * as yup from "yup";

@model({
  transportLayer: "/api/push_targets",
  permissionTarget: 'PushTarget',
  routingInfo: {
    create: "/settings/push_targets/create",
    update: "/settings/push_targets/:id/edit",
    read: "/settings/push_targets/:id"
  }
})
export class PushTargetModel extends Model {

  @attribute({ label: t`Module`, validation: yup.string().required(t`Module is a required field`) })
  app = "";

  @attribute({ label: t`Machine URL`, autoTable: false })
  machine = "";

  @attribute({ label: t`Machine State URL`, autoTable: false })
  machine_state = "";

  @attribute({ label: t`Job State URL`, autoTable: false })
  job_state = "";

  @attribute({ label: t`Error URL`, autoTable: false })
  error = "";

  @attribute({ label: t`Maintenance URL`, autoTable: false })
  maintenance = "";

  @attribute({ label: t`Event URL`, autoTable: false })
  event = "";

  @attribute({ label: t`Job Process Parameter URL`, autoTable: false })
  job_process_parameter = "";

  @attribute({ label: t`Order URL`, autoTable: false })
  order = "";

  @attribute({ label: t`Job URL`, autoTable: false })
  job = "";

  @attribute({ label: t`Order URL`, autoTable: false })
  block = "";

  @attribute({ label: t`Calibration URL`, autoTable: false })
  calibration = "";

  @attribute({ label: t`Box Coater Event URL`, autoTable: false })
  box_coater_event = "";

  @attribute({ label: t`Created At`, autoForm: false })
  created_at: Date = new Date();

  @attribute({ label: t`Updated At`, autoForm: false })
  updated_at: Date = new Date();

  constructor() {
    super();
    super.setup();
  }
}

export const PushTargetStore = PushTargetModel.store as DataStore<PushTargetModel>;
