class ForwardMachineDataJob < ApplicationJob
  queue_as :machine_data_forwarding

  def perform(endpoint, params, data)
    targets = MachineDataPushTarget.all
    targets.each do |target|
      url = "#{target.target_url}/connector/satisloh/#{endpoint[:url]}"
      Rails.logger.info "Forwarding to #{url}. Data => #{data}"
      response = Faraday.send(endpoint[:request_type], url) do |req|
        req.params['sn'] = params[:sn]
        req.params['date'] = params[:date]
        req.params['time'] = params[:time]
        req.headers['Content-Type'] = 'application/json'
        req.body = data.to_json
      end
      parsed_response = begin
                          Oj.load(response.body, symbol_keys: true)
                        rescue # If json parsing fails, pass through the body
                          response.body
                        end
      raise parsed_response[:error] || parsed_response[:err] || parsed_response.to_s || "Unknown error - consult logs" if response.status == 500
      parsed_response
    rescue => err
      Rails.logger.error "Failed to forward to #{url}. Data => : #{data}. Error: #{err}"
    end
  end
end
