class GenerateTemplateReportJob < ApplicationJob
  queue_as :report

  class InvalidReportTarget < StandardError; end

  def max_attempts
    # Reports are heavy, so don't use the default 25 retry
    3
  end

  def perform(report)
    report_id = report[:id]
    report_target = report[:target]
    report_params = JSON.parse(report[:params],{ symbolize_names: true })

    report_class = RxModule::Reporting.get_report(report_target)

    raise InvalidReportTarget if report_class.nil?

    update_report_status(report_id, 'running')

    report_results = report_class.new(report_params).run

    RxModule::Communicator.call_module_method('rx_reports', "#{report_url_for_template(report_id)}/attach_results", :post, {result: report_results})
  rescue StandardError => e
    Rails.logger.warn "Report generation ran into one or more issues: #{e}"
    update_report_status(report_id, 'failed', e)
  end

  def update_report_status(id, status, msg=nil)
    RxModule::Communicator.call_module_method('rx_reports', "#{report_url_for_template(id)}/update_status", :put, {status:status, message: msg})
  end

  def report_url_for_template(id)
    "#{reports_base_url}/#{id}"
  end

  def reports_base_url
    '/api/template_reports'
  end
end
