module FindHelpers
  module JobFinders
    class << self
      include NormalizeJobNumber

      def all_jobs_with_number(job_number)
        job_number = normalize_job_number(job_number)
        Job.where(number: job_number).order("expiry_date DESC")
      end

      # Returns active or expired job that matches number and are attached to an order, regardless of side
      def all_jobs_attached_to_order_with_number(job_number)
        job_number = normalize_job_number(job_number)
        Job.joins(:order).where(number: job_number).order("expiry_date DESC")
      end

      # Returns active or expired job that matches number, regardless of side
      def latest_with_number(job_number)
        all_jobs_with_number(job_number).first
      end

      # Returns active or expired job that matches number and is attached to an order, regardless of side
      def latest_attached_to_order_with_number(job_number)
        all_jobs_attached_to_order_with_number(job_number).first
      end

      # Returns first active job (left or right) - typically used as tray number
      def active_with_number(job_number)
        job_number = normalize_job_number(job_number)
        Job.active.where(number: job_number).order("expiry_date DESC").first
      end

      def find_active_or_create_by_number_and_side_num(job_number, side_num)
        job = active_with_number_and_side_num(job_number, side_num)
        if job.nil?
          job = Job.create(number: job_number, side: side_num)
        end
        job
      end

      # Returns the active job that matches number and side
      def active_with_number_and_side_num(job_number, side_num)
        job_number = normalize_job_number(job_number)
        Job.active.where(number: job_number, side: side_num).order("expiry_date DESC").first
      end

      # Returns the latest expired job that matches number and side
      def latest_expired_with_number_and_side_num(job_number, side_num)
        job_number = normalize_job_number(job_number)
        Job.expired.where(number: job_number, side: side_num).order("expiry_date DESC").first
      end

      def find_or_create_by_number_and_side_num(job_number, side_num)
        job = latest_with_number_and_side_num(job_number, side_num)
        if job.nil?
          job = Job.create(number: job_number, side: side_num)
        end
        job
      end

      # Returns active or expired job that matches both number and side
      def latest_with_number_and_side_num(job_number, side_num)
        all_jobs_with_number(job_number).where(side: side_num).first
      end
    end
  end
end
