module MachineKpiCalculator
  class BoxCoaterBatch < MachineKpiCalculator
    include KpiHelpers
    include BucketCalculator

    class << self
      def get_for_period_and_machines(start_time, end_time, timeframe, machine_ids, options: {})
        kpi = BoxCoaterBatch.new(start_time, end_time, timeframe, machine_ids, options)
        kpi.run
      end
    end

    def initialize(start_time, end_time, timeframe, machine_ids, options)
      super(start_time, end_time, timeframe, machine_ids, options)
    end

    def handlers
      [
        {
          key: :batch_between,
          handler: :batch_between,
          params: {}
        }
      ]
    end

    def aggregate_to_one_value?
      !!options[:aggregate_to_one_value]
    end

    def batch_between
      batch_per_day = time_buckets(start_time, end_time, timeframe, bucket_default_val: {
        value: 0,
        batch_on_this_day: 0
      })
      MachineKpi::EerKpiCalculator.timeline_calculator(start_time, end_time, machine_ids, false) do |timeline, event_indices_within_timeline, shift_start_time, shift_end_time|
        next if event_indices_within_timeline.empty?

        event_indices_within_timeline.each do |index|
          event = timeline[index]
          next unless event.event_type == "door_closed"

          batch = event.box_coater_batch
          next if batch.nil?

          time = time_bucket_index(event.time)
          batch_per_day.deep_merge!(
            {
              time => {
                value: Tools.percentage_maker(batch.lens_count, batch.lens_count + batch.empty_holes_count),
                batch_on_this_day: 1
              }
            }
          ) { |k, val1, val2| val1 + val2 }
        end
      end
      batch_per_day.each do |key, value|
        next batch_per_day[key] = 0 if value[:batch_on_this_day] == 0
        batch_per_day[key] = value[:value] / value[:batch_on_this_day]
      end
      batch_per_day
    end

    def build_results
      return data[:batch_between] unless aggregate_to_one_value?

      aggregate_to_one_value(data[:batch_between])
    end

    def build_results_per_machine
      data[:batch_between]
    end
  end
end
