module MachineKpiCalculator
  class BoxCoaterStateBreakdown < MachineKpiCalculator
    include KpiHelpers
    include BucketCalculator

    class << self
      def get_for_period_and_machines(start_time, end_time, timeframe, machine_ids, options: {})
        kpi = BoxCoaterStateBreakdown.new(start_time, end_time, timeframe, machine_ids, options)
        kpi.run
      end
    end

    def initialize(start_time, end_time, timeframe, machine_ids, options)
      super(start_time, end_time, timeframe, machine_ids, options)
    end

    def handlers
      [
        {
          key: :time_in_state_breakdown,
          handler: :time_in_state_breakdown,
          params: {}
        }
      ]
    end

    def as_percent?
      options[:as_percent]
    end

    def time_in_state_breakdown
      time_in_state = time_buckets(start_time, end_time, timeframe, bucket_default_val: {
        door_open: 0,
        door_closed: 0,
        engineering_lockout: 0
      })
      EerKpiCalculator.timeline_calculator(start_time, end_time, machine_ids, true) do |timeline, event_indices_within_timeline, shift_start_time, shift_end_time|

        next if event_indices_within_timeline.empty?

        # Loop over each event and add duration to hash
        event_indices_within_timeline.each do |index|
          event = timeline[index]
          duration = EerKpiCalculator.event_duration_in_shift(event, shift_start_time, shift_end_time)
          time_in_state[time_bucket_index(shift_start_time)][event.event_type.to_sym] += duration
        end
      end

      time_in_state
    end

    def build_results
      bucket = data[:time_in_state_breakdown]
      # Maybe add a method in hub to get those activeTime between multiple intervals
      bucket.each do |time, value|
        end_of_bucket = time + timeframe_length
        if RxModule::Communicator::Shifts.shifts_between(time, end_of_bucket)[:models].blank?
          time_in_shift = timeframe_length * machine_ids.size
        else
          time_in_shift = RxModule::Communicator::Shifts.active_time_between(time, end_of_bucket) * machine_ids.size
        end
        active_time = time_in_shift * machine_ids.size
        bucket[time][:active_time] = active_time
        if as_percent?
          lockout_percent = 0
          open_percent = 0
          closed_percent = 0
          unless active_time == 0
            lockout_percent = Tools.percentage_maker(value[:engineering_lockout], active_time)
            open_percent = Tools.percentage_maker(value[:door_open], active_time)
            closed_percent = Tools.percentage_maker(value[:door_closed], active_time)
          end
          bucket[time][:engineering_lockout] = lockout_percent
          bucket[time][:door_open] = open_percent
          bucket[time][:door_closed] = closed_percent
        end
      end
      bucket
    end

    def build_results_per_machine
      data[:time_in_state_breakdown]
    end
  end
end
