module MachineKpiCalculator
  class FinishedJobRatio < MachineKpiCalculator

    include KpiTableHandler

    class << self
      def get_for_period_and_machines(start_time, end_time, timeframe, machine_ids, options: {})
        kpi = FinishedJobRatio.new(start_time, end_time, timeframe, machine_ids, options)
        kpi.run
      end
    end

    def initialize(start_time, end_time, timeframe, machine_ids, options)
      super(start_time, end_time, timeframe, machine_ids, options)
    end

    def handlers
      [
        {
          key: :job_states,
          handler: KpiTableHandler::JobStateCountKpiHandler,
          params: {
            job_states: %w[finished aborted],
            use_pair: ProductionSetting.metric_is_pair?
          }
        }
      ]
    end

    def build_results
      data[:job_states].each do |k, v|
        aborted = (v[:aborted] || 0)
        finished = (v[:finished] || 0)
        processed = aborted + finished

        if processed == 0
          data[:job_states][k] = 0
        else
          data[:job_states][k] = ((finished / processed.to_f)*100)
        end
      end
    end
  end
end