module MachineKpiCalculator
  class Uptime < MachineKpiCalculator

    include KpiTableHandler

    class << self
      def get_for_period_and_machines(start_time, end_time, timeframe, machine_ids, options)
        kpi = Uptime.new(start_time, end_time, timeframe, machine_ids, options)
        kpi.run
      end
    end

    def initialize(start_time, end_time, timeframe, machine_ids, options)
      super(start_time, end_time, timeframe, machine_ids, options)
    end

    def handlers
      [
        {
          key: :machine_state_duration,
          handler: KpiTableHandler::MachineStateDurationKpiHandler,
          params: {
            machine_states: %w[processing waitingforjobs],
            sum_as: 'uptime'
          }
        }
      ]
    end

    # Returns a hash of time buckets and the sum of time in 'uptime' by all specified machines
    # Utilization is expressed in a number of seconds unless 'as_percent' is passed in as an option
    # in which case the value returned is a percentage value.
    # ex:
    # {
    #   {13h00 01/01/2022 : 1600}
    #   {14h00 01/01/2022 : 747}
    # }
    def build_results
      data[:machine_state_duration].each do |k,v|
        data[:machine_state_duration][k] = v[:uptime]
      end
    end

    def build_results_per_machine
      data[:machine_state_duration].each do |machine_id, buckets|
        buckets.each do |k,v|
          data[:machine_state_duration][machine_id][k] = v[:uptime]
        end
      end
    end
  end
end