# == Schema Information
#
# Table name: colors
#
#  id          :bigint           not null, primary key
#  color       :string(4000)
#  status_name :string(4000)
#  created_at  :datetime         not null
#  updated_at  :datetime         not null
#
class Color < ApplicationRecord
  class << self
    def apply_default_colors
      Color.find_or_create_by(status_name: 'ON').update(color: "#ffffc4")
      Color.find_or_create_by(status_name: 'READY').update(color: "#0000ff")
      Color.find_or_create_by(status_name: 'WAITINGFORJOBS').update(color:  "#00ff00")
      Color.find_or_create_by(status_name: 'PROCESSING').update(color: "#008000")
      Color.find_or_create_by(status_name: 'NOTREADY').update(color: "#ffa000")
      Color.find_or_create_by(status_name: 'ERROR').update(color: "#ff0000")
      Color.find_or_create_by(status_name: 'HALT').update(color: "#ffa0a0")
      Color.find_or_create_by(status_name: 'ESTOP').update(color: "#ff0080")
      Color.find_or_create_by(status_name: 'MAINTENANCE').update(color: "#00c4ff")
      Color.find_or_create_by(status_name: 'CALIBRATION').update(color: "#ff5000")
      Color.find_or_create_by(status_name: 'SETUP').update(color: "#ffff00")
      Color.find_or_create_by(status_name: 'OFF').update(color: "#000000")
      Color.find_or_create_by(status_name: 'NOTCONNECTED').update(color: "#808080")
    end

    def apply_simplified_colors
      dark_green = '#008000'
      light_green = '#80FF00'
      red = '#FF0000'
      orange = '#FFC000'
      black = '#000000'

      Color.find_or_create_by(status_name: 'ON').update(color: orange)
      Color.find_or_create_by(status_name: 'READY').update(color: orange)
      Color.find_or_create_by(status_name: 'WAITINGFORJOBS').update(color: dark_green)
      Color.find_or_create_by(status_name: 'PROCESSING').update(color: light_green)
      Color.find_or_create_by(status_name: 'NOTREADY').update(color: red)
      Color.find_or_create_by(status_name: 'ERROR').update(color: red)
      Color.find_or_create_by(status_name: 'HALT').update(color: red)
      Color.find_or_create_by(status_name: 'ESTOP').update(color: red)
      Color.find_or_create_by(status_name: 'MAINTENANCE').update(color: orange)
      Color.find_or_create_by(status_name: 'CALIBRATION').update(color: orange)
      Color.find_or_create_by(status_name: 'SETUP').update(color: orange)
      Color.find_or_create_by(status_name: 'OFF').update(color: black)
      Color.find_or_create_by(status_name: 'NOTCONNECTED').update(color: black)
    end
    
    def seed
      return puts "Colors already seeded. Skipping seeding." if Color.count > 0

      puts "Seeding Colors..."

      ProductionSetting.use_simple_machine_state_colors? ? apply_simplified_colors : apply_default_colors

      puts "Seeding Colors... [COMPLETED]"
    end
  end
end
