module BoxCoater
  module MachineHelper
    extend ActiveSupport::Concern

    def is_box_coater?
      return false if machine_type.nil?
      machine_type.category == 'box_coater'
    end

    def current_coater_event
      self.box_coater_events.order(time: :desc).first
    end

    def is_in_engineering_lockout?
      current_event = current_coater_event
      return false if current_event.nil?

      current_event.is_engineering_lockout?
    end

    def start_engineering_lockout
      current_event = current_coater_event
      return if current_event.is_engineering_lockout?

      # Close previous event before starting the lockout
      current_event.update(duration: Time.zone.now - current_event.time) unless current_event.nil?

      BoxCoaterEvent.create(machine: self, time: Time.zone.now, event_type: 'engineering_lockout')
    end

    def stop_engineering_lockout
      current_event = current_coater_event
      return unless current_event.is_engineering_lockout?

      current_event.update(duration: Time.zone.now - current_event.time)

      # Start a new box coater event using the most recent actual door state
      door_latest_state = get_latest_door_state

      # If the machine has no door event somehow, assume the door is open
      if door_latest_state.nil? || !door_latest_state.is_box_coater_event?
        BoxCoaterEvent.create(machine: self, time: Time.zone.now, event_type: 'door_open')
      else
        if door_latest_state.is_door_open?
          BoxCoaterEvent.create(machine: self, time: Time.zone.now, event_type: 'door_open')
        else
          # If the last even is a door open, we also need to create a new batch
          event = BoxCoaterEvent.new(machine: self, time: Time.zone.now, event_type: 'door_closed')
          event.box_coater_batch = BoxCoaterBatch.new(machine: self, time: Time.zone.now, box_coater_event: event)
          event.save
        end
      end
    end

    private

    # Gets the latest door state based on machine state data (by pass box coater events)
    # Required to reset the latest door state after a engineering lockout
    def get_latest_door_state
      machine_states = MachineState.arel_table
      MachineState.where(machine: self).order(time: :desc).where(machine_states[:comment].matches("%door%")).first
    end
  end
end

