module BoxCoater
  module MachineStateHelper
    extend ActiveSupport::Concern

    def is_box_coater_event?
      machine.track_eer_metrics && (is_door_state? || has_recipe?)
    end

    def has_recipe?
      return false if addon.nil?
      addon.include? "recipe="
    end

    def recipe
      return nil unless has_recipe?
      addon.downcase.split('recipe=')[1]
    end

    def is_door_state?
      return false if comment.nil?
      is_door_closed? || is_door_open?
    end

    def is_door_open?
      return false if comment.nil?
      comment.downcase.include?("door open")
    end

    def is_door_closed?
      return false if comment.nil?
      comment.downcase.include?("door closed")
    end

    def box_coater_event_type
      if is_door_open?
        'door_open'
      elsif is_door_closed?
        'door_closed'
      else
        nil
      end
    end
  end
end
