module Cloneable
  extend ActiveSupport::Concern

  class_methods do
    def associations_to_clone
      []
    end

    def cloneable?
      true
    end

    def copy_to_db(id, from:, to:)
      model = ActiveRecord::Base.connected_to(shard: from, role: :reading) do
        find_by(id)&.deep_clone(include: associations_to_clone)
      end
      return if model.nil?

      ActiveRecord::Base.connected_to(shard: to, role: :writing) do
        model.save
      end
    end

    def copy_to_archive(id)
      copy_to_db(id, from: :primary, to: :archive)
    end

    def copy_to_primary(id)
      copy_to_db(id, from: :archive, to: :primary)
    end
  end
end
