module Expirable
  extend ActiveSupport::Concern

  included do
    before_create do
      if expiry_date.nil?
        self.expiry_date = created_at + DataArchiveSetting.instance.job_max_age.hour
      end
    end
  end

  class_methods do
    def active
      where(arel_table[:expiry_date].gt(Time.now))
    end

    def expired
      where(arel_table[:expiry_date].lt(Time.now))
    end
  end

  def expired?
    Time.now > expiry_date
  end

  def reset_expiry
    update(expiry_date: Time.now + DataArchiveSetting.instance.job_max_age.hour)
  end

  def expire
    update(expiry_date: DateTime.now - 1.second)
  end
end