# == Schema Information
#
# Table name: eer_yields
#
#  id         :bigint           not null, primary key
#  date       :datetime
#  value      :integer
#  created_at :datetime         not null
#  updated_at :datetime         not null
#  machine_id :bigint
#
# Indexes
#
#  index_eer_yields_on_machine_id  (machine_id)
#
class EerYield < ApplicationRecord
  belongs_to :machine

  reportable("eer_yield", "Yield Report", "Report on eer yields", {
    value: {
      name: "Yield",
      desc: "Yield (%) on a given day"
    },
    date: {
      name: "Date",
      desc: "Day on which the yield was reported for"
    },
    machine: {
      name: 'Machine',
      desc: "Machine associated with this yield"
    }
  })

  def self.yields_between(start_time, end_time, machine_ids)
    EerYield.where(machine_id: machine_ids)
                    .where(EerYield.arel_table[:date].gteq(start_time)
                                         .and(EerYield.arel_table[:date].lt(end_time)))
                    .order(date: :asc)
  end
end