# == Schema Information
#
# Table name: events
#
#  id         :bigint           not null, primary key
#  event_type :integer
#  machine_sn :string(4000)
#  message    :string(4000)
#  time       :datetime
#  created_at :datetime         not null
#  updated_at :datetime         not null
#
# Indexes
#
#  index_events_on_machine_sn  (machine_sn)
#
# Foreign Keys
#
#  fk_rails_...  (machine_sn => machines.sn) ON DELETE => cascade
#
class Event < ApplicationRecord
  include Pushable
  include ByMachineAndPeriod
  include ForwardMachineData

  belongs_to :machine, foreign_key: :machine_sn, primary_key: :sn, touch: true
  has_many :event_data

  def forwarding_data
    {
      message: message,
      type: event_type,
      eventData: {
        data: event_data.map {|e_data|
          {
            name: e_data.name,
            value: e_data.value
          }
        }
      }
    }
  end

  def forwarding_endpoint
    {
      request_type: :put,
      url:     'event/1.0/add'
    }
  end

  def forwarding_params
    {
      sn: machine_sn,
      date: time.strftime("%Y-%m-%d"),
      time: time.strftime("%H:%M:%S,%L")
    }
  end

  def self.delete_before(date)
    data_archive_settings = DataArchiveSetting.first
    max_delete_queue = data_archive_settings.delete_count_event
    return 0 if max_delete_queue.nil? || max_delete_queue <= 0
    deleted_objects = 0
    # Delete the events that have existed before the given date
    # Only delete a maximum of `max_delete_queue` events
    ActiveRecord::Base.connected_to(shard: :primary, role: :writing) do
      events = Event.where(Event.arel_table[:time].lt(date)).limit(max_delete_queue)
      deleted_objects += events.delete_all
    end
    deleted_objects
  end
end
