# == Schema Information
#
# Table name: job_processes
#
#  id                       :bigint           not null, primary key
#  side                     :integer
#  time                     :datetime
#  created_at               :datetime         not null
#  updated_at               :datetime         not null
#  batch_id                 :string(4000)
#  job_id                   :bigint           not null
#  machine_id               :bigint           not null
#  manufacturing_process_id :bigint
#
# Indexes
#
#  index_job_processes_on_job_id                    (job_id)
#  index_job_processes_on_machine_id                (machine_id)
#  index_job_processes_on_manufacturing_process_id  (manufacturing_process_id)
#  index_job_processes_on_side                      (side)
#
# Foreign Keys
#
#  fk_rails_...  (job_id => jobs.id) ON DELETE => cascade
#  fk_rails_...  (machine_id => machines.id) ON DELETE => cascade
#  fk_rails_...  (manufacturing_process_id => manufacturing_processes.id) ON DELETE => cascade
#
class JobProcess < ApplicationRecord
  include Pushable
  include Sideable
  include ForwardMachineData
  include BoxCoater::JobProcessHelper

  belongs_to :machine, touch: true
  belongs_to :job, touch: true
  belongs_to :manufacturing_process, required: false, touch: true
  has_many :job_process_parameters

  def forwarding_data
    {
      job: job.number,
      lensside: side_letter,
      process: manufacturing_process.nil? ? nil : manufacturing_process.name,
      batchId: batch_id,
      params: {
        param: job_process_parameters.map {|process_params|
          {
            name: process_params.name,
            value: process_params.value
          }
        }
      }
    }
  end

  def forwarding_endpoint
    {
      request_type: :put,
      url: 'processparameter/1.1/add'
    }
  end

  def forwarding_params
    {
      sn: machine.sn,
      date: time.strftime("%Y-%m-%d"),
      time: time.strftime("%H:%M:%S,%L")
    }
  end
end
