# == Schema Information
#
# Table name: kpi_machine_error_counts
#
#  id         :bigint           not null, primary key
#  count      :integer          default(0)
#  group_code :string(4000)     not null
#  time       :datetime         not null
#  created_at :datetime         not null
#  updated_at :datetime         not null
#  machine_id :bigint
#
# Indexes
#
#  index_kpi_machine_error_counts_on_machine_id                 (machine_id)
#  kpi_machine_error_count_uniq_time_per_machine_or_group_code  (time,machine_id,group_code) UNIQUE
#
# Foreign Keys
#
#  fk_rails_...  (machine_id => machines.id) ON DELETE => cascade
#
class KpiMachineErrorCount < ApplicationRecord
  include KpiHelpers

  belongs_to :machine

  def should_archive?
    false
  end

  def self.no_filter_prefix
    "unknown-"
  end

  def self.error_group_code(error)
    if error.has_filter?
      group_code = error.error_filter.group_code
    else
      group_code = "#{KpiMachineErrorCount.no_filter_prefix}#{error.errno}"
    end
    group_code
  end

  def self.cache_error(error)
    group_code = error_group_code(error)

    cache_entry = KpiHelpers.get_kpi_cache_entry(KpiMachineErrorCount, error.time, { machine_id: error.machine.id, group_code: group_code})

    if cache_entry.nil?
      cache_entry = KpiMachineErrorCount.create(
        time: KpiHelpers.kpi_bucket_time(error.time),
        machine_id: error.machine.id,
        group_code: group_code,
        count: 1
      )
    else
      cache_entry.update(count: cache_entry.count += 1)
    end

    cache_entry
  end

  def self.build_default_cache_entry(bucket_time, machine_id, group_code)
    {
      group_code: group_code,
      count: 0,
      time: bucket_time,
      machine_id: machine_id,
      created_at: Time.zone.now,
      updated_at: Time.zone.now
    }
  end

  def self.backwards_cache_entry_calculation(entry)
    # TODO: Modify function to be able to do multiple buckets
    start_time = entry[:time]
    end_time = start_time + 30.minutes
    machine_id = entry[:machine_id]
    machine_sn = Machine.find_by(id: machine_id).sn

    cache_entries = []
    errors = Error.where(machine_sn: machine_sn).where(Error.arel_table[:time].gteq(start_time).and(Error.arel_table[:time]).lt(end_time))

    if errors.count == 0
      cache_entries << build_default_cache_entry(start_time,  machine_id, 'none')
    else
      errors.find_each do |error|
        group_code = error_group_code(error)

        entry = cache_entries.find{|e| e[:group_code] == group_code}

        if entry.nil?
          entry = build_default_cache_entry(start_time,  machine_id, group_code)
          cache_entries << entry
        end

        entry[:count] += 1
      end
    end

    cache_entries
  end
end
