# == Schema Information
#
# Table name: maintenances
#
#  id               :bigint           not null, primary key
#  machine_sn       :string(4000)
#  maintenance      :string(4000)
#  maintenance_type :integer
#  status           :integer
#  time             :datetime
#  created_at       :datetime         not null
#  updated_at       :datetime         not null
#
# Indexes
#
#  index_maintenances_on_machine_sn  (machine_sn)
#
# Foreign Keys
#
#  fk_rails_...  (machine_sn => machines.sn) ON DELETE => cascade
#
class Maintenance < ApplicationRecord
  include Pushable
  include ByMachineAndPeriod
  include ForwardMachineData

  enum maintenance_type: [:unknown, :counter, :date, :"time period", :event]
  enum status: [:raised, :acknowledged, :finished]
  belongs_to :machine, foreign_key: :machine_sn, primary_key: :sn, touch: true

  def forwarding_data
    {
      type: maintenance_type,
      status: status,
      maintenance: maintenance,
    }
  end

  def forwarding_endpoint
    {
      request_type: :put,
      url: 'maintenance/1.0/add'
    }
  end

  def forwarding_params
    {
      sn: machine_sn,
      date: time.strftime("%Y-%m-%d"),
      time: time.strftime("%H:%M:%S,%L")
    }
  end

  def self.delete_before(date)
    data_archive_settings = DataArchiveSetting.first
    max_delete_queue = data_archive_settings.delete_count_maintenance
    return 0 if max_delete_queue.nil? || max_delete_queue <= 0
    deleted_objects = 0
    # Delete the maintenances that have existed before the given date
    # Only delete a maximum of `max_delete_queue` maintenances
    ActiveRecord::Base.connected_to(shard: :primary, role: :writing) do
      maintenances = Maintenance.where(Maintenance.arel_table[:time].lt(date)).limit(max_delete_queue)
      deleted_objects += maintenances.delete_all
    end
    deleted_objects
  end
end
