module ReportComponentHelpers
  extend ActiveSupport::Concern

  def color_for_series(count)
    series_colors[0..count]
  end

  def series_colors
    %w[#008FFB #00E396 #FEB019 #FF4560 #775DD0 #3F51B5 #03A9F4 #4CAF50 #F9CE1D #FF9800]
  end

  def build_series_for_kpi_bucket(name, data, type, formatter)
    {
      name: name,
      data: kpi_entries_to_dataset(data, formatter),
      type: type,
    }
  end

  def build_series_for_kpi_buckets_per_machine(buckets_per_machines, type, formatter)
    buckets_per_machines.map do |machine_id, buckets|
      machine = Machine.find_by_id(machine_id)
      build_series_for_kpi_bucket(machine&.ensured_name || machine_id, buckets, type, formatter)
    end
  end

  def kpi_entries_to_dataset(data, formatter)
    data.map do |time, result|
      {
        x: bucket_time_to_timeframe_label(time),
        y: send(formatter, result)
      }
    end
  end

  def bucket_time_to_timeframe_label(time)
    #TODO: remove @use_reduced_timeframe once everything is ported over to v3
    case @use_reduced_timeframe ? reduce_report_timeframe : timeframe
    when KpiHelpers::Timeframe::HOUR_BY_HOUR
      return "#{time.strftime('%H:%M')} - #{(time + 1.hour).strftime('%H:%M')}"
    when KpiHelpers::Timeframe::DAY_BY_DAY
      return "#{time.strftime('%d/%m')}"
    when KpiHelpers::Timeframe::WEEK_BY_WEEK
      "#{time.strftime('%d/%m')} - #{(time + 1.week).strftime('%d/%m')}"
    end
  end

  # The timeframe used by this component is one 'less' than the overall report (if we're running a daily, we need an hourly breakdown)
  def reduce_report_timeframe
    raise InvalidTimeFrame if timeframe - 1 < 0
    timeframe - 1
  end

  # If one or more machine target collects error, this component can be used
  def machine_targets_collect_error(machine_ids)
    collects_error = 0

    Machine.where(id: machine_ids).find_each do |machine|
      if machine.has_machine_type?
        collects_error += 1 if machine.machine_type.error_enabled
      else
        collects_error += 1 # assume that machine with no type may collect errors
      end
    end

    collects_error > 0
  end
end