module ReportComponents
  class MachineEfficiencyOverview < ReportComponent
    include DataHelpers
    include ReportComponentHelpers

    def component_id
      'machine_efficiency_overview'
    end

    def component_type
      'line'
    end

    def generate(args)
      oee = MachineKpi.overall_equipment_effectiveness_between(start_time, end_time, timeframe, machine_ids, options: args)
      mtbf = MachineKpi.mean_time_between_failures_between(start_time, end_time, timeframe, machine_ids, options: {})
      mttr = MachineKpi.mean_time_to_restore_between(start_time, end_time, timeframe, machine_ids, options: {})

      oee_target_value = args[:oee_target]
      oee_series = oee[:buckets].map do |machine_id, buckets|
        machine = Machine.find_by_id(machine_id)
        {
          name: machine&.ensured_name || machine_id,
          data: buckets.values,
          type: 'line'
        }
      end
      # All timestamps are the same, so get one of machine_ids, then get the times from that bucket
      oee_labels = oee[:buckets][oee[:buckets].keys[0]].keys.map {|k| bucket_time_to_timeframe_label(k)}
      oee_target_array = oee_labels.map {|label| oee_target_value}

      oee_series << {
        name: 'target',
        data: oee_target_array,
        type: 'line'
      }

      {
        breakdown: {
          name: 'breakdown',
          value: oee_series,
          labels: oee_labels,
          colors: args[:colorize_series] ?
            color_for_series(oee_series.size) :
            nil
        },
        oee: {
          name: 'OEE',
          value: "#{oee[:value]}%"
        },
        yield: {
          name: 'Yield (%)',
          value: "#{oee[:yield]}%"
        },
        load: {
          name: 'Load (%)',
          value: "#{oee[:load]}%"
        },
        uptime: {
          name: 'Uptime (%)',
          value: "#{oee[:uptime]}%"
        },
        mtbf: {
          name: 'MTBF',
          value: "#{(mtbf.to_f / 1.minute).round(0)} min"
        },
        mttr: {
          name: 'MTTR',
          value: "#{(mttr.to_f / 1.minute).round(0)} min"
        }
      }
    end
  end
end