module ReportComponents
  class MachineTopAbortsBar < ReportComponent
    include ReportComponentHelpers
    include KpiHelpers

    def component_id
      'machine_top_aborts'
    end

    def component_type
      'bar'
    end

    def generate(args)
      kpi_top_aborts = MachineKpi.top_aborts_between(start_time, end_time, Timeframe::ALL, machine_ids, options: args)
      categories = if args[:aggregate_machines]
                    kpi_top_aborts.map {|kpi| kpi[:code]}
                  else
                    kpi_top_aborts.values[0].map {|code, count| code}
                  end

      data = {
        categories: categories,
        series: if args[:aggregate_machines]
                  build_series_for_top_aborts_aggregated(kpi_top_aborts)
                else
                  build_series_for_top_aborts_per_machine(kpi_top_aborts)
                end
      }

      return set_no_data('No aborts during reporting period') if data[:series].size == 0 || data[:series][0][:data].size == 0

      data[:colors] = color_for_series(kpi_top_aborts.size) if args[:colorize_series]

      data
    end

    def build_series_for_top_aborts_aggregated(top_aborts_kpi)
      data = top_aborts_kpi.map do |top_abort_kpi|
        top_abort_kpi[:count]
      end
      return [] if data.size == 0
      [{
         data: data
       }]
    end

    def build_series_for_top_aborts_per_machine(top_aborts_kpi)
      top_aborts_kpi.map do |machine_id, aborts|
        machine = Machine.find_by_id(machine_id)
        {
          name: machine&.ensured_name || machine_id,
          data: aborts.map {|code, count| count}
        }
      end
    end
  end
end