module ReportComponents
  class RejectStationKickouts < ReportComponent
    include DataHelpers
    include ReportComponentHelpers

    def component_id
      'reject_station_kickouts'
    end

    def component_type
      'line'
    end

    def generate(args)
      job_state_machine_ids = Machine.where(id: machine_ids).map {|reject_machine| reject_machine.machine_ids}.flatten.uniq

      @kickouts = MachineKpi.aggregated_kickouts_between_per_machine(start_time, end_time, timeframe, machine_ids, options: {})
      @job_states = MachineKpi.started_between_per_machine(start_time, end_time, timeframe, job_state_machine_ids, options: {})

      #Using own build series because the ReportComponentHelper one doesnt pass machine_id, and we need it to get the number of job_states from attached machines.
      {
        series: build_series
      }
    end

    def build_series
      @kickouts.map do |reject_id, hash|
        reject_machine = Machine.where(id: reject_id).first
        {
          data: hash.map do |time, kickout|
            job_state_machine_ids = reject_machine.machine_ids
            job_state_sum = job_state_machine_ids.reduce(0) {|sum, id| sum + @job_states[id][time]}
            {
              y: y_calc(kickout[:rejects], job_state_sum),
              x: bucket_time_to_timeframe_label(time)
            }
          end,
          name: reject_machine.name || reject_machine.sn
        }
      end
    end

    def y_calc(numerator, denominator)
      if denominator.nil? && numerator > 0
        y = 100.0
      elsif denominator.nil? || denominator == 0
        y = 0.0
      else
        y = (numerator.to_f/denominator.to_f) * 100
      end
      y.round(0)
    end
  end
end