class Rack::Attack
  ### Throttle Spammy Clients ###

  def self.rate_limit
    return ENV["API_RATE_LIMIT"].to_i unless ENV["API_RATE_LIMIT"].nil?
    2000
  end

  # Throttle all requests by IP (300rpm)
  #
  # Key: "rack::attack:#{Time.now.to_i/:period}:req/ip:#{req.ip}"
  throttle("req/ip", limit: rate_limit, period: 1.minutes) do |req|
    req.ip
  end

  # Whitelist modules
  # This is obviously not super secure, but it's just the rate limit so
  # I'm sure it's fine
  Rack::Attack.safelist("mark any module safe") do |request|
    # Requests are allowed if the return value is truthy
    request.env["HTTP_X_FROM_MODULE"].present?
  end

  ### Custom Throttle Response ###

  self.throttled_response = lambda do |env|
    match_data = env["rack.attack.match_data"]
    now = match_data[:epoch_time]

    headers = {
      "X-Rate-Limit-Limit" => match_data[:limit].to_s,
      "X-Rate-Limit-Remaining" => "0",
      "X-Rate-Limit-Reset" => (now + (match_data[:period] - now % match_data[:period])).to_s,
      "Content-Type" => "application/json"
    }
    Rails.logger.info({ message: "Request throttled", data: match_data })
    return [429, headers, ['{"error": "Too Many Requests. Retry Later."}']]
  end
end
