def seed_file_and_db(file_name, table, columns, max_numbers, &block)
  unless Rails.root.join("test", "performance", file_name).exist?
    time = Time.parse("2022-03-10T0:00:00,000").in_time_zone(ENV['TZ'])
    CSV.open(Rails.root.join("test", "performance", file_name), "w", row_sep: "\r\n") do |csv|
      csv << columns
      (1..max_numbers[2]).each do |i|
        if i == max_numbers[1]+1
          time += 2.week
        elsif i == max_numbers[2]+1
          time += 2.month
        end
        block.call(i, time, csv)
      end
    end
  end

  sql = <<~SQL
    BULK INSERT #{table}
    FROM '#{Rails.root.join("test", "performance", file_name)}'
    WITH 
    (
      FIELDTERMINATOR = ',',
      FIRSTROW = 2,
      ROWTERMINATOR = '\n',
      KEEPNULLS,
      DATAFILETYPE = 'char'
    )
  SQL
  ActiveRecord::Base.connection.execute(sql)
end

DataArchiveSetting.first.update(job_max_age: 99999999)
require 'CSV'
string_format = "%Y-%m-%d %k:%M:%S"
number_of_machines = 108
number_of_machine_states_per_day = number_of_machines * 4800
number_of_machine_states_per_month = number_of_machine_states_per_day * 31
number_of_machine_states_per_3_months = number_of_machine_states_per_month * 3 #48211200

number_of_jobs_per_day = 15000
number_of_jobs_per_month = number_of_jobs_per_day * 31
number_of_jobs_per_3_months = number_of_jobs_per_month * 3 #1395000

number_of_job_states_per_day = 9600 * number_of_machines
number_of_job_states_per_month = number_of_job_states_per_day * 31
number_of_job_states_per_3_months = number_of_job_states_per_month * 3 #96422400

number_of_job_data_per_day = 300000
number_of_job_data_per_month = number_of_job_data_per_day * 31
number_of_job_data_per_3_months = number_of_job_data_per_month * 3 #27900000

#Process_params doesnt make sense to fetch all for a month, so lets cap at 100 million
number_of_process_params_per_day = 8000000
number_of_process_params_per_month = 64000000
number_of_process_params_per_3_months = 100000000

number_of_blocks_per_day = 10000
number_of_blocks_per_month = number_of_blocks_per_day * 31
number_of_blocks_per_3_months = number_of_blocks_per_month * 3

machine_time = Time.parse("2022-03-10T0:00:00,000").in_time_zone(ENV['TZ'])
@machines = (1..number_of_machines).map do |i|
  {sn: i, name: i, created_at: machine_time, updated_at: machine_time}
end
Machine.insert_all!(@machines)

seed_file_and_db("jobs.csv", 'jobs', %w[id number name description sequence_number priority manufacturer side lmat_id lmat_type batch_criteria template job_type enabled order_id created_at updated_at expiry_date], [number_of_jobs_per_day, number_of_jobs_per_month, number_of_jobs_per_3_months]) do |i, time, csv|
  csv << [
    i,
    i,
    i,
    nil,
    0,
    0,
    nil,
    0,
    nil,
    nil,
    nil,
    nil,
    nil,
    nil,
    nil,
    time.utc.strftime(string_format),
    time.utc.strftime(string_format),
    (time+2.year).utc.strftime(string_format)
  ]
end

seed_file_and_db("job_states.csv", 'job_states', %w[id job_id batch_id process side state comment scrap scrap_reason time created_at updated_at machine_id substate], [number_of_job_states_per_day, number_of_job_states_per_month, number_of_job_states_per_3_months]) do |i, time, csv|
  csv << [
    i,
    (i/70)+1,
    nil,
    nil,
    0,
    nil,
    nil,
    nil,
    nil,
    time.utc.strftime(string_format),
    time.utc.strftime(string_format),
    time.utc.strftime(string_format),
    (i/892800)+1,
    nil
  ]
end

seed_file_and_db('machine_states.csv', 'machine_states',%w[id machine_sn state time created_at updated_at comment addon], [number_of_machine_states_per_day, number_of_machine_states_per_month, number_of_machine_states_per_3_months]) do |i, time, csv|
  csv << [
    i,
    (i/446400)+1,
    nil,
    time.utc.strftime(string_format),
    time.utc.strftime(string_format),
    time.utc.strftime(string_format),
    nil,
    nil
  ]
end

seed_file_and_db('job_data.csv', 'job_data', %w[id name value side job_id created_at updated_at], [number_of_job_data_per_day, number_of_job_data_per_month, number_of_job_data_per_3_months]) do |i, time, csv|
  csv << [
    i,
    i,
    i,
    0,
    (i/20)+1,
    time.utc.strftime(string_format),
    time.utc.strftime(string_format)
  ]
end

seed_file_and_db('job_process.csv', 'job_processes', %w[id machine_id job_id manufacturing_process_id batch_id time created_at updated_at side], [number_of_process_params_per_day, number_of_process_params_per_month, number_of_process_params_per_3_months]) do |i, time, csv|
  csv << [
    i,
    i%108,
    i%number_of_jobs_per_day,
    nil,
    nil,
    time.utc.strftime(string_format),
    time.utc.strftime(string_format),
    time.utc.strftime(string_format),
    nil
  ]
end

seed_file_and_db('job_process_parameter.csv', 'job_process_parameters', %w[id job_process_id name value created_at updated_at], [number_of_process_params_per_day, number_of_process_params_per_month, number_of_process_params_per_3_months]) do |i, time, csv|
  csv << [
    i,
    i,
    i,
    i,
    time.utc.strftime(string_format),
    time.utc.strftime(string_format),
  ]
end

seed_file_and_db('blocks.csv', 'blocks', %w[id dmc side rfid_tag usage_count job_id created_at updated_at cycles_until_wash cycles_until_discard], [number_of_blocks_per_day, number_of_blocks_per_month, number_of_blocks_per_3_months]) do |i, time, csv|
  csv << [
    i,
    i,
    i%2,
    i,
    i,
    i%number_of_jobs_per_day,
    time.utc.strftime(string_format),
    time.utc.strftime(string_format),
    nil,
    nil
  ]
end