require 'swagger_helper'

def description_override(value = nil)
  return super() if value.nil?

  metadata[:operation][:description] = value
end

def get_api_description_override(value)
  value << "
### A maximum of 20 machine serial numbers and 20 job numbers can be passed in a single request.
Because of these limitations, the post endpoint should be used whenever possible."
  description_override(value)
end

RSpec.describe 'external_api/api', type: :request do
  path "/machine_states" do
    post "Fetch Machine States" do
      tags "Machine"
      description_override "### Get machine states related to machines."
      consumes 'application/json'
      produces 'application/json'
      parameter name: :data, required: true, in: :body, schema: {
        '$ref' => '#/components/schemas/machine_state_body'
      }
      response "200", "machine states fetched" do
        schema '$ref' => '#components/schemas/machine_state_resp'
        run_test!
      end
      response "422", "invalid request" do
        run_test!
      end
    end

    get "Fetch Machine States (Using post is recommended)" do
      tags "Machine"
      get_api_description_override "### Get machine states related to machines."
      consumes 'application/json'
      produces 'application/json'
      parameter name: :start, description: "Start time of the target time range", required: true, in: :query, schema: {
        '$ref' => '#/components/schemas/time'
      }
      parameter name: :end, description: "End time of the target time range", required: true, in: :query, schema: {
        '$ref' => '#/components/schemas/time'
      }
      parameter name: "sn[]", description: "Serial numbers of the targeted machines", required: true, in: :query, schema: {
        '$ref' => '#/components/schemas/sns',
        explode: true,
      }
      parameter name: "state[]", in: :query, schema: {
        '$ref' => '#/components/schemas/machine_states',
        explode: true,
      }
      parameter name: :limit, in: :query, schema: {
        '$ref' => '#/components/schemas/limit'
      }
      parameter name: :offset, in: :query, schema: {
        '$ref' => '#/components/schemas/offset'
      }
      parameter name: :order, in: :query, schema: {
        '$ref' => '#/components/schemas/order'
      }
      response "200", "Machine states fetched" do
        schema '$ref' => '#components/schemas/machine_state_resp'
        run_test!
      end
      response "422", "invalid request" do
        run_test!
      end
    end
  end

  path "/job_states" do
    post "Fetch Job States" do
      tags "Job"
      description_override "### Get job states related to jobs/machines
\\*One *sn* or *job* is required.
##### Due to job numbers being typically reused, it is necessary to provide a date range to successfully identify a job."
      consumes 'application/json'
      produces 'application/json'
      parameter name: :data, required: true, in: :body, schema: {
        '$ref' => '#/components/schemas/job_state_body',
      }
      response "200", "job states fetched" do
        schema '$ref' => '#components/schemas/job_state_resp'
        run_test!
      end
      response "422", "invalid request" do
        run_test!
      end
    end

    get "Fetch Job States (Using post is recommended)" do
      tags "Job"
      get_api_description_override "### Get job states related to jobs/machines
\\*One *sn* or *job* is required.
##### Due to job numbers being typically reused, it is necessary to provide a date range to successfully identify a job."
      consumes 'application/json'
      produces 'application/json'
      parameter name: :start, description: "Start time of the target time range", required: true, in: :query, schema: {
        '$ref' => '#/components/schemas/time'
      }
      parameter name: :end, description: "End time of the target time range", required: true, in: :query, schema: {
        '$ref' => '#/components/schemas/time'
      }
      parameter name: "sn[]", description: "Serial numbers of the targeted machines", in: :query, schema: {
        '$ref' => '#/components/schemas/sns',
        explode: true,
      }
      parameter name: "job[]", description: "Job numbers for the targeted job", in: :query, schema: {
        '$ref' => '#/components/schemas/jobs',
        explode: true,
      }
      parameter name: "state[]", in: :query, schema: {
        '$ref' => '#/components/schemas/job_states'
      }
      parameter name: :side, in: :query, schema: {
        '$ref' => '#/components/schemas/side'
      }
      parameter name: :process, in: :query, schema: {
        '$ref' => '#/components/schemas/processes'
      }
      parameter name: :limit, in: :query, schema: {
        '$ref' => '#/components/schemas/limit'
      }
      parameter name: :offset, in: :query, schema: {
        '$ref' => '#/components/schemas/offset'
      }
      parameter name: :order, in: :query, schema: {
        '$ref' => '#/components/schemas/order'
      }
      response "200", "Job states fetched" do
        schema '$ref' => '#components/schemas/job_state_resp'
        run_test!
      end
      response "422", "invalid request" do
        run_test!
      end
    end
  end

  path "/process_params" do
    post "Fetch Process parameters" do
      tags "Machine"
      description_override "### Get process parameters related to jobs and/or machines.
*One *sn* or *job* is required."
      consumes 'application/json'
      produces 'application/json'
      parameter name: :data, required: true, in: :body, schema: {
        '$ref' => '#/components/schemas/process_params_body'
      }
      response "200", "process parameters fetched" do
        schema '$ref' => '#components/schemas/process_params_resp'
        run_test!
      end
      response "422", "invalid request" do
        run_test!
      end
    end

    get "Fetch process params (Using post is recommended)" do
      tags "Machine"
      get_api_description_override "### Get process parameters related to jobs and/or machines.
*One *sn* or *job* is required."
      consumes 'application/json'
      produces 'application/json'
      parameter name: :start, description: "Start time of the target time range", required: true, in: :query, schema: {
        '$ref' => '#/components/schemas/time'
      }
      parameter name: :end, description: "End time of the target time range", required: true, in: :query, schema: {
        '$ref' => '#/components/schemas/time'
      }
      parameter name: "sn[]", description: "Serial numbers of the targeted machines", in: :query, schema: {
        '$ref' => '#/components/schemas/sns',
        explode: true,
      }
      parameter name: "job[]", description: "Job numbers for the targeted job", in: :query, schema: {
        '$ref' => '#/components/schemas/jobs',
        explode: true,
      }
      parameter name: "param[]", description: "Name of targeted parameter", in: :query, schema: {
        '$ref' => '#/components/schemas/params',
        explode: true,
      }
      parameter name: :side, in: :query, schema: {
        '$ref' => '#/components/schemas/side',
      }
      parameter name: :limit, in: :query, schema: {
        '$ref' => '#/components/schemas/limit'
      }
      parameter name: :offset, in: :query, schema: {
        '$ref' => '#/components/schemas/offset'
      }
      parameter name: :order, in: :query, schema: {
        '$ref' => '#/components/schemas/order'
      }
      response "200", "Process params fetched" do
        schema '$ref' => '#components/schemas/process_params_resp'
        run_test!
      end
      response "422", "invalid request" do
        run_test!
      end
    end
  end

  path "/machine_errors" do
    post "Fetch Machine Errors" do
      tags "Machine"
      description_override "### Get machine errors related to specific machines."
      consumes 'application/json'
      produces 'application/json'
      parameter name: :data, required: true, in: :body, schema: {
        '$ref' => '#/components/schemas/machine_errors_body'
      }
      response "200", "Machine errors fetched" do
        schema '$ref' => '#components/schemas/machine_errors_resp'
        run_test!
      end
      response "422", "invalid request" do
        run_test!
      end
    end

    get "Fetch machine errors (Using post is recommended)" do
      tags "Machine"
      get_api_description_override "### Get machine errors related to specific machines."
      consumes 'application/json'
      produces 'application/json'
      parameter name: :start, description: "Start time of the target time range", required: true, in: :query, schema: {
        '$ref' => '#/components/schemas/time'
      }
      parameter name: :end, description: "End time of the target time range", required: true, in: :query, schema: {
        '$ref' => '#/components/schemas/time'
      }
      parameter name: "sn[]", description: "Serial numbers of the targeted machines", required: true, in: :query, schema: {
        '$ref' => '#/components/schemas/sns',
        explode: true,
      }
      parameter name: "job[]", description: "Job numbers for the targeted job", in: :query, schema: {
        '$ref' => '#/components/schemas/jobs',
        explode: true,
      }
      parameter name: :lesside, in: :query, schema: {
        '$ref' => '#/components/schemas/side',
      }
      parameter name: :errno, in: :query, schema: {
        '$ref' => '#/components/schemas/errno',
      }
      parameter name: :errortype, in: :query, schema: {
        '$ref' => '#/components/schemas/errortype',
      }
      parameter name: :status, in: :query, schema: {
        '$ref' => '#/components/schemas/status',
      }
      parameter name: :limit, in: :query, schema: {
        '$ref' => '#/components/schemas/limit'
      }
      parameter name: :offset, in: :query, schema: {
        '$ref' => '#/components/schemas/offset'
      }
      parameter name: :order, in: :query, schema: {
        '$ref' => '#/components/schemas/order'
      }
      response "200", "Machine errors fetched" do
        schema '$ref' => '#components/schemas/machine_errors_resp'
        run_test!
      end
      response "422", "invalid request" do
        run_test!
      end
    end
  end

  path "/lens_attribute" do
    post "Fetch Lens Data" do
      tags "Job"
      description_override "### Get Job Data from specific jobs or time frames or both.
*Start and end are required unless a job is specified."
      consumes 'application/json'
      produces 'application/json'
      parameter name: :data, required: true, in: :body, schema: {
        '$ref' => '#/components/schemas/lens_attribute_body'
      }
      response "200", "Lens attributes fetched" do
        schema '$ref' => '#components/schemas/lens_attribute_resp'
        run_test!
      end
      response "422", "invalid request" do
        run_test!
      end
    end

    get "Fetch Lens Data (Using post is recommended)" do
      tags "Job"
      get_api_description_override "### Get Job Data from specific jobs or time frames or both.
*Start and end are required unless a job is specified."
      consumes 'application/json'
      produces 'application/json'
      parameter name: :start, description: "Start time of the target time range", in: :query, schema: {
        '$ref' => '#/components/schemas/time'
      }
      parameter name: :end, description: "End time of the target time range", in: :query, schema: {
        '$ref' => '#/components/schemas/time'
      }
      parameter name: "job[]", description: "Job numbers for the targeted job", in: :query, schema: {
        '$ref' => '#/components/schemas/jobs',
        explode: true,
      }
      parameter name: :side, in: :query, schema: {
        '$ref' => '#/components/schemas/side',
      }
      parameter name: :attribute, in: :query, schema: {
        '$ref' => '#/components/schemas/attribute_names',
        explode: true
      }
      parameter name: :limit, in: :query, schema: {
        '$ref' => '#/components/schemas/limit'
      }
      parameter name: :offset, in: :query, schema: {
        '$ref' => '#/components/schemas/offset'
      }
      parameter name: :order, in: :query, schema: {
        '$ref' => '#/components/schemas/order'
      }
      response "200", "Lens attributes fetched" do
        schema '$ref' => '#components/schemas/lens_attribute_resp'
        run_test!
      end
      response "422", "invalid request" do
        run_test!
      end
    end
  end

  path "/block_pieces" do
    post "Fetch Block Pieces" do
      tags "Block"
      description_override "### Get Blocks from Rx-Data using the their last use date.
##### Blocks do not explicitly have a time property, the last use time of the block will be used to identify the relevant data"
      consumes 'application/json'
      produces 'application/json'
      parameter name: :data, required: true, in: :body, schema: {
        '$ref' => '#/components/schemas/block_pieces_body'
      }
      response "200", "Block pieces fetched" do
        schema '$ref' => '#components/schemas/block_pieces_resp'
        run_test!
      end
      response "422", "invalid request" do
        run_test!
      end
    end

    get "Fetch Block Pieces (Using post is recommended)" do
      tags "Block"
      get_api_description_override "### Get Blocks from Rx-Data using the their last use date.
##### Blocks do not explicitly have a time property, the last use time of the block will be used to identify the relevant data"
      consumes 'application/json'
      produces 'application/json'
      parameter name: :start, description: "Start time of the target time range", required: true, in: :query, schema: {
        '$ref' => '#/components/schemas/time'
      }
      parameter name: :end, description: "End time of the target time range", required: true, in: :query, schema: {
        '$ref' => '#/components/schemas/time'
      }
      parameter name: :limit, in: :query, schema: {
        '$ref' => '#/components/schemas/limit'
      }
      parameter name: :offset, in: :query, schema: {
        '$ref' => '#/components/schemas/offset'
      }
      parameter name: :order, in: :query, schema: {
        '$ref' => '#/components/schemas/order'
      }
      response "200", "Block pieces fetched" do
        schema '$ref' => '#components/schemas/block_pieces_resp'
        run_test!
      end
      response "422", "invalid request" do
        run_test!
      end
    end
  end
end